@@ -20,14 +20,14 @@
  */
 /*
   File:      actions.cpp
-  Version:   $Rev: 1458 $
+  Version:   $Rev: 1692 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   08-Dec-03, ahu: created
              30-Apr-06, Roger Larsson: Print filename if processing multiple files
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: actions.cpp 1458 2008-05-22 08:38:20Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: actions.cpp 1692 2008-12-08 05:32:02Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -51,6 +51,7 @@ EXIV2_RCSID("@(#) $Id: actions.cpp 1458 2008-05-22 08:38:20Z ahuggel $")
 #include "canonmn.hpp"
 #include "iptc.hpp"
 #include "xmp.hpp"
+#include "preview.hpp"
 #include "futils.hpp"
 #include "i18n.h"                // NLS support.
 
@@ -97,14 +98,16 @@ namespace {
         time_t modtime_;
     };
 
-    // Convert a string "YYYY:MM:DD HH:MI:SS" to a struct tm type,
-    // returns 0 if successful
+    /*!
+      @brief Convert a string "YYYY:MM:DD HH:MI:SS" to a struct tm type,
+             returns 0 if successful
+     */
     int str2Tm(const std::string& timeStr, struct tm* tm);
 
-    // Convert a UTC time to a string "YYYY:MM:DD HH:MI:SS", "" on error
+    //! Convert a UTC time to a string "YYYY:MM:DD HH:MI:SS", "" on error
     std::string time2Str(time_t time);
 
-    // Convert a tm structure to a string "YYYY:MM:DD HH:MI:SS", "" on error
+    //! Convert a tm structure to a string "YYYY:MM:DD HH:MI:SS", "" on error
     std::string tm2Str(struct tm* tm);
 
     /*!
@@ -137,8 +140,9 @@ namespace {
     /*!
       @brief Make a file path from the current file path, destination
              directory (if any) and the filename extension passed in.
+
       @param path Path of the existing file
-      @param ext New filename extension (incl. the dot '.' if required)
+      @param ext  New filename extension (incl. the dot '.' if required)
       @return 0 if successful, 1 if the new file exists and the user
              chose not to overwrite it.
      */
@@ -146,7 +150,7 @@ namespace {
 
     /*!
       @brief Check if file \em path exists and whether it should be
-             overwritten. Ask user if necessary. Return 1 if the file 
+             overwritten. Ask user if necessary. Return 1 if the file
              exists and shouldn't be overwritten, else 0.
      */
     int dontOverwrite(const std::string& path);
@@ -220,11 +224,10 @@ namespace Action {
         path_ = path;
         int rc = 0;
         switch (Params::instance().printMode_) {
-        case Params::pmSummary:     rc = printSummary(); break;
-        case Params::pmList:        rc = printList(); break;
-        case Params::pmIptc:        rc = printIptc(); break;
-        case Params::pmXmp:         rc = printXmp(); break;
-        case Params::pmComment:     rc = printComment(); break;
+        case Params::pmSummary: rc = printSummary();     break;
+        case Params::pmList:    rc = printList();        break;
+        case Params::pmComment: rc = printComment();     break;
+        case Params::pmPreview: rc = printPreviewList(); break;
         }
         return rc;
     }
@@ -258,7 +261,7 @@ namespace Action {
             std::cout << buf.st_size << " " << _("Bytes") << std::endl;
         }
 
-        // MIME type 
+        // MIME type
         printLabel(_("MIME type"));
         std::cout << image->mimeType() << "\n";
 
@@ -328,6 +331,9 @@ namespace Action {
         if (!done) {
             done = 0 != printTag(exifData, "Exif.Olympus.FlashBias");
         }
+        if (!done) {
+            done = 0 != printTag(exifData, "Exif.OlympusCs.FlashExposureComp");
+        }
         std::cout << std::endl;
 
         // Actual focal length and 35 mm equivalent
@@ -338,7 +344,8 @@ namespace Action {
             md = exifData.findKey(
                 Exiv2::ExifKey("Exif.Photo.FocalLengthIn35mmFilm"));
             if (md != exifData.end()) {
-                std::cout << " ("<< _("35 mm equivalent") << ": " << *md << ")";
+                std::cout << " ("<< _("35 mm equivalent") << ": "
+                          << md->print(&exifData) << ")";
             }
         }
         std::cout << std::endl;
@@ -427,6 +434,9 @@ namespace Action {
         if (!done) {
             done = 0 != printTag(exifData, "Exif.Olympus.Macro");
         }
+        if (!done) {
+            done = 0 != printTag(exifData, "Exif.OlympusCs.MacroMode");
+        }
         if (!done) {
             done = 0 != printTag(exifData, "Exif.Panasonic.Macro");
         }
@@ -463,6 +473,9 @@ namespace Action {
         if (!done) {
             done = 0 != printTag(exifData, "Exif.Olympus.Quality");
         }
+        if (!done) {
+            done = 0 != printTag(exifData, "Exif.OlympusCs.Quality");
+        }
         if (!done) {
             done = 0 != printTag(exifData, "Exif.Panasonic.Quality");
         }
@@ -491,14 +504,14 @@ namespace Action {
         if (md == exifData.end()) {
             md = exifData.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
         }
-        if (md != exifData.end()) {
+        if (md != exifData.end() && md->count() > 0) {
             xdim = md->toLong();
         }
         md = exifData.findKey(Exiv2::ExifKey("Exif.Image.ImageLength"));
         if (md == exifData.end()) {
             md = exifData.findKey(Exiv2::ExifKey("Exif.Photo.PixelYDimension"));
         }
-        if (md != exifData.end()) {
+        if (md != exifData.end() && md->count() > 0) {
             ydim = md->toLong();
         }
         if (xdim != 0 && ydim != 0) {
@@ -531,6 +544,9 @@ namespace Action {
         if (!done) {
             done = 0 != printTag(exifData, "Exif.Olympus.WhiteBalance");
         }
+        if (!done) {
+            done = 0 != printTag(exifData, "Exif.OlympusCs.WhiteBalance");
+        }
         if (!done) {
             done = 0 != printTag(exifData, "Exif.Panasonic.WhiteBalance");
         }
@@ -546,18 +562,27 @@ namespace Action {
         if (!done) {
             done = 0 != printTag(exifData, "Exif.MinoltaCsOld.WhiteBalance");
         }
+        if (!done) {
+            done = 0 != printTag(exifData, "Exif.Photo.WhiteBalance");
+        }
         std::cout << std::endl;
 
         // Thumbnail
         printLabel(_("Thumbnail"));
-        std::string thumbExt = exifData.thumbnailExtension();
+        Exiv2::ExifThumbC exifThumb(exifData);
+        std::string thumbExt = exifThumb.extension();
         if (thumbExt.empty()) {
             std::cout << _("None");
         }
         else {
-            Exiv2::DataBuf buf = exifData.copyThumbnail();
-            std::cout << exifData.thumbnailFormat() << ", "
-                      << buf.size_ << " " << _("Bytes");
+            Exiv2::DataBuf buf = exifThumb.copy();
+            if (buf.size_ == 0) {
+                std::cout << _("None");
+            }
+            else {
+                std::cout << exifThumb.mimeType() << ", "
+                          << buf.size_ << " " << _("Bytes");
+            }
         }
         std::cout << std::endl;
 
@@ -592,7 +617,7 @@ namespace Action {
         Exiv2::ExifKey ek(key);
         Exiv2::ExifData::const_iterator md = exifData.findKey(ek);
         if (md != exifData.end()) {
-            std::cout << *md;
+            md->write(std::cout, &exifData);
             rc = 1;
         }
         if (!label.empty()) std::cout << std::endl;
@@ -601,199 +626,174 @@ namespace Action {
 
     int Print::printList()
     {
+        int rc = 0;
         if (!Exiv2::fileExists(path_, true)) {
             std::cerr << path_
                       << ": " << _("Failed to open the file\n");
-            return -1;
+            rc = -1;
         }
         Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(path_);
         assert(image.get() != 0);
         image->readMetadata();
-        Exiv2::ExifData& exifData = image->exifData();
-        if (exifData.empty()) {
-            std::cerr << path_
-                      << ": " << _("No Exif data found in the file\n");
-            return -3;
-        }
-        Exiv2::ExifData::const_iterator md;
         bool const manyFiles = Params::instance().files_.size() > 1;
-        for (md = exifData.begin(); md != exifData.end(); ++md) {
-            if (   Params::instance().unknown_
-                && md->tagName().substr(0, 2) == "0x") {
-                continue;
-            }
-            if (manyFiles) {
-                std::cout << std::setfill(' ') << std::left << std::setw(20)
-                          << path_ << "  ";
-            }
-            bool first = true;
-            if (Params::instance().printItems_ & Params::prTag) {
-                if (!first) std::cout << " ";
-                first = false;
-                std::cout << "0x" << std::setw(4) << std::setfill('0')
-                          << std::right << std::hex
-                          << md->tag();
-            }
-            if (Params::instance().printItems_ & Params::prGroup) {
-                if (!first) std::cout << " ";
-                first = false;
-                std::cout << std::setw(12) << std::setfill(' ') << std::left
-                          << md->groupName();
-            }
-            if (Params::instance().printItems_ & Params::prKey) {
-                if (!first) std::cout << " ";
-                first = false;
-                std::cout << std::setfill(' ') << std::left << std::setw(44)
-                          << md->key();
-            }
-            if (Params::instance().printItems_ & Params::prName) {
-                if (!first) std::cout << " ";
-                first = false;
-                std::cout << std::setw(27) << std::setfill(' ') << std::left
-                          << md->tagName();
-            }
-            if (Params::instance().printItems_ & Params::prLabel) {
-                if (!first) std::cout << " ";
-                first = false;
-                std::cout << std::setw(30) << std::setfill(' ') << std::left
-                          << md->tagLabel();
-            }
-            if (Params::instance().printItems_ & Params::prType) {
-                if (!first) std::cout << " ";
-                first = false;
-                std::cout << std::setw(9) << std::setfill(' ') << std::left
-                          << md->typeName();
-            }
-            if (Params::instance().printItems_ & Params::prCount) {
-                if (!first) std::cout << " ";
-                first = false;
-                std::cout << std::dec << std::setw(3)
-                          << std::setfill(' ') << std::right
-                          << md->count();
-            }
-            if (Params::instance().printItems_ & Params::prSize) {
-                if (!first) std::cout << " ";
-                first = false;
-                std::cout << std::dec << std::setw(3)
-                          << std::setfill(' ') << std::right
-                          << md->size();
-            }
-            if (Params::instance().printItems_ & Params::prValue) {
-                if (!first) std::cout << "  ";
-                first = false;
-                if (   Params::instance().binary_
-                    && md->typeId() == Exiv2::undefined
-                    && md->size() > 100) {
-                    std::cout << _("(Binary value suppressed)") << std::endl;
-                    continue;
-                }
-                std::cout << std::dec << md->value();
-            }
-            if (Params::instance().printItems_ & Params::prTrans) {
-                if (!first) std::cout << "  ";
-                first = false;
-                if (   Params::instance().binary_
-                    && md->typeId() == Exiv2::undefined
-                    && md->size() > 100) {
-                    std::cout << _("(Binary value suppressed)") << std::endl;
-                    continue;
-                }
-                std::cout << std::dec << *md;
-            }
-            if (Params::instance().printItems_ & Params::prHex) {
-                if (!first) std::cout << std::endl;
-                first = false;
-                if (   Params::instance().binary_
-                    && md->typeId() == Exiv2::undefined
-                    && md->size() > 100) {
-                    std::cout << _("(Binary value suppressed)") << std::endl;
-                    continue;
-                }
-                Exiv2::DataBuf buf(md->size());
-                md->copy(buf.pData_, exifData.byteOrder());
-                Exiv2::hexdump(std::cout, buf.pData_, buf.size_);
-            }
-            std::cout << std::endl;
+        // Set defaults for metadata types and data columns
+        if (Params::instance().printTags_ == Exiv2::mdNone) {
+            Params::instance().printTags_ = Exiv2::mdExif | Exiv2::mdIptc | Exiv2::mdXmp;
         }
-
-        return 0;
+        if (Params::instance().printItems_ == 0) {
+            Params::instance().printItems_ = Params::prKey | Params::prType | Params::prCount | Params::prTrans;
+        }
+        if (Params::instance().printTags_ & Exiv2::mdExif) {
+            Exiv2::ExifData& exifData = image->exifData();
+            for (Exiv2::ExifData::const_iterator md = exifData.begin();
+                 md != exifData.end(); ++md) {
+                printMetadatum(*md, image.get(), manyFiles);
+            }
+            if (exifData.empty()) {
+                if (Params::instance().verbose_) {
+                    std::cerr << path_ << ": " << _("No Exif data found in the file\n");
+                }
+                rc = -3;
+            }
+        }
+        if (Params::instance().printTags_ & Exiv2::mdIptc) {
+            Exiv2::IptcData& iptcData = image->iptcData();
+            for (Exiv2::IptcData::const_iterator md = iptcData.begin();
+                 md != iptcData.end(); ++md) {
+                printMetadatum(*md, image.get(), manyFiles);
+            }
+            if (iptcData.empty()) {
+                if (Params::instance().verbose_) {
+                    std::cerr << path_ << ": " << _("No IPTC data found in the file\n");
+                }
+                rc = -3;
+            }
+        }
+        if (Params::instance().printTags_ & Exiv2::mdXmp) {
+            Exiv2::XmpData& xmpData = image->xmpData();
+            for (Exiv2::XmpData::const_iterator md = xmpData.begin();
+                 md != xmpData.end(); ++md) {
+                printMetadatum(*md, image.get(), manyFiles);
+            }
+            if (xmpData.empty()) {
+                if (Params::instance().verbose_) {
+                    std::cerr << path_ << ": " << _("No XMP data found in the file\n");
+                }
+                rc = -3;
+            }
+        }
+        return rc;
     } // Print::printList
 
-    int Print::printIptc()
+    void Print::printMetadatum(const Exiv2::Metadatum& md,
+                               const Exiv2::Image* pImage,
+                               bool const manyFiles)
     {
-        if (!Exiv2::fileExists(path_, true)) {
-            std::cerr << path_
-                      << ": " << _("Failed to open the file\n");
-            return -1;
+        if (   Params::instance().unknown_
+            && md.tagName().substr(0, 2) == "0x") {
+            return;
         }
-        Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(path_);
-        assert(image.get() != 0);
-        image->readMetadata();
-        Exiv2::IptcData& iptcData = image->iptcData();
-        if (iptcData.empty()) {
-            std::cerr << path_
-                      << ": " << _("No IPTC data found in the file\n");
-            return -3;
+        if (manyFiles) {
+            std::cout << std::setfill(' ') << std::left << std::setw(20)
+                      << path_ << "  ";
         }
-        Exiv2::IptcData::const_iterator end = iptcData.end();
-        Exiv2::IptcData::const_iterator md;
-        bool manyFiles = Params::instance().files_.size() > 1;
-        for (md = iptcData.begin(); md != end; ++md) {
-            std::cout << std::setfill(' ') << std::left;
-            if (manyFiles) {
-                std::cout << std::setw(20) << path_ << " ";
+        bool first = true;
+        if (Params::instance().printItems_ & Params::prTag) {
+            if (!first) std::cout << " ";
+            first = false;
+            std::cout << "0x" << std::setw(4) << std::setfill('0')
+                      << std::right << std::hex
+                      << md.tag();
+        }
+        if (Params::instance().printItems_ & Params::prGroup) {
+            if (!first) std::cout << " ";
+            first = false;
+            std::cout << std::setw(12) << std::setfill(' ') << std::left
+                      << md.groupName();
+        }
+        if (Params::instance().printItems_ & Params::prKey) {
+            if (!first) std::cout << " ";
+            first = false;
+            std::cout << std::setfill(' ') << std::left << std::setw(44)
+                      << md.key();
+        }
+        if (Params::instance().printItems_ & Params::prName) {
+            if (!first) std::cout << " ";
+            first = false;
+            std::cout << std::setw(27) << std::setfill(' ') << std::left
+                      << md.tagName();
+        }
+        if (Params::instance().printItems_ & Params::prLabel) {
+            if (!first) std::cout << " ";
+            first = false;
+            std::cout << std::setw(30) << std::setfill(' ') << std::left
+                      << md.tagLabel();
+        }
+        if (Params::instance().printItems_ & Params::prType) {
+            if (!first) std::cout << " ";
+            first = false;
+            std::cout << std::setw(9) << std::setfill(' ') << std::left;
+            const char* tn = md.typeName();
+            if (tn) {
+                std::cout << tn;
             }
-            std::cout << std::setw(44)
-                      << md->key() << " "
-                      << std::setw(9) << std::setfill(' ') << std::left
-                      << md->typeName() << " "
-                      << std::dec << std::setw(3)
-                      << std::setfill(' ') << std::right
-                      << md->count() << "  "
-                      << std::dec << md->value()
-                      << std::endl;
-        }
-
-        return 0;
-    } // Print::printIptc
-
-    int Print::printXmp()
-    {
-        if (!Exiv2::fileExists(path_, true)) {
-            std::cerr << path_
-                      << ": " << _("Failed to open the file\n");
-            return -1;
-        }
-        Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(path_);
-        assert(image.get() != 0);
-        image->readMetadata();
-        Exiv2::XmpData& xmpData = image->xmpData();
-        if (xmpData.empty()) {
-            std::cerr << path_
-                      << ": " << _("No XMP data found in the file\n");
-            return -3;
-        }
-        Exiv2::XmpData::const_iterator end = xmpData.end();
-        Exiv2::XmpData::const_iterator md;
-        bool manyFiles = Params::instance().files_.size() > 1;
-        for (md = xmpData.begin(); md != end; ++md) {
-            std::cout << std::setfill(' ') << std::left;
-            if (manyFiles) {
-                std::cout << std::setw(20) << path_ << " ";
+            else {
+                std::ostringstream os;
+                os << "0x" << std::setw(4) << std::setfill('0') << std::hex << md.typeId();
+                std::cout << os.str();
             }
-            std::cout << std::setw(44)
-                      << md->key() << " "
-                      << std::setw(9) << std::setfill(' ') << std::left
-                      << md->typeName() << " "
-                      << std::dec << std::setw(3)
-                      << std::setfill(' ') << std::right
-                      << md->count() << "  "
-                      << std::dec << *md
-                      << std::endl;
         }
-
-        return 0;
-    } // Print::printXmp
+        if (Params::instance().printItems_ & Params::prCount) {
+            if (!first) std::cout << " ";
+            first = false;
+            std::cout << std::dec << std::setw(3)
+                      << std::setfill(' ') << std::right
+                      << md.count();
+        }
+        if (Params::instance().printItems_ & Params::prSize) {
+            if (!first) std::cout << " ";
+            first = false;
+            std::cout << std::dec << std::setw(3)
+                      << std::setfill(' ') << std::right
+                      << md.size();
+        }
+        if (Params::instance().printItems_ & Params::prValue) {
+            if (!first) std::cout << "  ";
+            first = false;
+            if (   Params::instance().binary_
+                && md.typeId() == Exiv2::undefined
+                && md.size() > 100) {
+                std::cout << _("(Binary value suppressed)") << std::endl;
+                return;
+            }
+            std::cout << std::dec << md.value();
+        }
+        if (Params::instance().printItems_ & Params::prTrans) {
+            if (!first) std::cout << "  ";
+            first = false;
+            if (   Params::instance().binary_
+                && md.typeId() == Exiv2::undefined
+                && md.size() > 100) {
+                std::cout << _("(Binary value suppressed)") << std::endl;
+                return;
+            }
+            std::cout << std::dec << md.print(&pImage->exifData());
+        }
+        if (Params::instance().printItems_ & Params::prHex) {
+            if (!first) std::cout << std::endl;
+            first = false;
+            if (   Params::instance().binary_
+                && md.typeId() == Exiv2::undefined
+                && md.size() > 100) {
+                std::cout << _("(Binary value suppressed)") << std::endl;
+                return;
+            }
+            Exiv2::DataBuf buf(md.size());
+            md.copy(buf.pData_, pImage->byteOrder());
+            Exiv2::hexdump(std::cout, buf.pData_, buf.size_);
+        }
+        std::cout << std::endl;
+    } // Print::printMetadatum
 
     int Print::printComment()
     {
@@ -812,6 +812,36 @@ namespace Action {
         return 0;
     } // Print::printComment
 
+    int Print::printPreviewList()
+    {
+        if (!Exiv2::fileExists(path_, true)) {
+            std::cerr << path_
+                      << ": " << _("Failed to open the file\n");
+            return -1;
+        }
+        Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(path_);
+        assert(image.get() != 0);
+        image->readMetadata();
+        bool const manyFiles = Params::instance().files_.size() > 1;
+        int cnt = 0;
+        Exiv2::PreviewManager pm(*image);
+        Exiv2::PreviewPropertiesList list = pm.getPreviewProperties();
+        for (Exiv2::PreviewPropertiesList::const_iterator pos = list.begin(); pos != list.end(); ++pos) {
+            if (manyFiles) {
+                std::cout << std::setfill(' ') << std::left << std::setw(20)
+                          << path_ << "  ";
+            }
+            std::cout << _("Preview") << " " << ++cnt << ": "
+                      << pos->mimeType_ << ", ";
+            if (pos->width_ != 0 && pos->height_ != 0) {
+                std::cout << pos->width_ << "x" << pos->height_ << " "
+                          << _("pixels") << ", ";
+            }
+            std::cout << pos->size_ << " " << _("bytes") << "\n";
+        }
+        return 0;
+    } // Print::printPreviewList
+
     Print::AutoPtr Print::clone() const
     {
         return AutoPtr(clone_());
@@ -958,15 +988,14 @@ namespace Action {
 
     int Erase::eraseThumbnail(Exiv2::Image* image) const
     {
-        Exiv2::ExifData& exifData = image->exifData();
-        std::string thumbExt = exifData.thumbnailExtension();
+        Exiv2::ExifThumb exifThumb(image->exifData());
+        std::string thumbExt = exifThumb.extension();
         if (thumbExt.empty()) {
             return 0;
         }
-        long delta = exifData.eraseThumbnail();
+        exifThumb.erase();
         if (Params::instance().verbose_) {
-            std::cout << _("Erasing") << " " << delta
-                        << " " << _("Bytes of thumbnail data") << std::endl;
+            std::cout << _("Erasing thumbnail data") << std::endl;
         }
         return 0;
     }
@@ -1029,8 +1058,12 @@ namespace Action {
             if (dontOverwrite(xmpPath)) return 0;
             rc = metacopy(path_, xmpPath, Exiv2::ImageType::xmp, false);
         }
+        if (Params::instance().target_ & Params::ctPreview) {
+            rc = writePreviews();
+        }
         if (   !(Params::instance().target_ & Params::ctXmpSidecar)
-            && !(Params::instance().target_ & Params::ctThumb)) {
+            && !(Params::instance().target_ & Params::ctThumb)
+            && !(Params::instance().target_ & Params::ctPreview)) {
             std::string exvPath = newFilePath(path_, ".exv");
             if (dontOverwrite(exvPath)) return 0;
             rc = metacopy(path_, exvPath, Exiv2::ImageType::exv, false);
@@ -1061,7 +1094,8 @@ namespace Action {
             return -3;
         }
         int rc = 0;
-        std::string thumbExt = exifData.thumbnailExtension();
+        Exiv2::ExifThumb exifThumb(exifData);
+        std::string thumbExt = exifThumb.extension();
         if (thumbExt.empty()) {
             std::cerr << path_ << ": " << _("Image does not contain an Exif thumbnail\n");
         }
@@ -1070,20 +1104,76 @@ namespace Action {
             std::string thumbPath = thumb + thumbExt;
             if (dontOverwrite(thumbPath)) return 0;
             if (Params::instance().verbose_) {
-                Exiv2::DataBuf buf = exifData.copyThumbnail();
-                std::cout << _("Writing") << " "
-                          << exifData.thumbnailFormat() << " " << _("thumbnail") << " ("
-                          << buf.size_ << " " << _("Bytes") << ") " << _("to file") << " "
-                          << thumbPath << std::endl;
+                Exiv2::DataBuf buf = exifThumb.copy();
+                if (buf.size_ != 0) {
+                    std::cout << _("Writing thumbnail") << " (" << exifThumb.mimeType() << ", "
+                              << buf.size_ << " " << _("Bytes") << ") " << _("to file") << " "
+                              << thumbPath << std::endl;
+                }
             }
-            rc = exifData.writeThumbnail(thumb);
-            if (rc) {
+            rc = exifThumb.writeFile(thumb);
+            if (rc == 0) {
                 std::cerr << path_ << ": " << _("Exif data doesn't contain a thumbnail\n");
             }
         }
         return rc;
     } // Extract::writeThumbnail
 
+    int Extract::writePreviews() const
+    {
+        if (!Exiv2::fileExists(path_, true)) {
+            std::cerr << path_
+                      << ": " << _("Failed to open the file\n");
+            return -1;
+        }
+        Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(path_);
+        assert(image.get() != 0);
+        image->readMetadata();
+
+        Exiv2::PreviewManager pvMgr(*image);
+        Exiv2::PreviewPropertiesList pvList = pvMgr.getPreviewProperties();
+
+        const Params::PreviewNumbers& numbers = Params::instance().previewNumbers_;
+        for (Params::PreviewNumbers::const_iterator n = numbers.begin(); n != numbers.end(); ++n) {
+            if (*n == 0) {
+                // Write all previews
+                for (int num = 0; num < static_cast<int>(pvList.size()); ++num) {
+                    writePreviewFile(pvMgr.getPreviewImage(pvList[num]), num + 1);
+                }
+                break;
+            }
+            if (*n > static_cast<int>(pvList.size())) {
+                std::cerr << path_ << ": " << _("Image does not have preview")
+                          << " " << *n << "\n";
+                continue;
+            }
+            writePreviewFile(pvMgr.getPreviewImage(pvList[*n - 1]), *n);
+        }
+        return 0;
+    } // Extract::writePreviews
+
+    void Extract::writePreviewFile(const Exiv2::PreviewImage& pvImg, int num) const
+    {
+        std::string pvFile = newFilePath(path_, "-preview") + Exiv2::toString(num);
+        std::string pvPath = pvFile + pvImg.extension();
+        if (dontOverwrite(pvPath)) return;
+        if (Params::instance().verbose_) {
+            std::cout << _("Writing preview") << " " << num << " ("
+                      << pvImg.mimeType() << ", ";
+            if (pvImg.width() != 0 && pvImg.height() != 0) {
+                std::cout << pvImg.width() << "x" << pvImg.height() << " "
+                          << _("pixels") << ", ";
+            }
+            std::cout << pvImg.size() << " " << _("bytes") << ") "
+                      << _("to file") << " " << pvPath << std::endl;
+        }
+        long rc = pvImg.writeFile(pvFile);
+        if (rc == 0) {
+            std::cerr << path_ << ": " << _("Image does not have preview")
+                      << " " << num << "\n";
+        }
+    } // Extract::writePreviewFile
+
     Extract::AutoPtr Extract::clone() const
     {
         return AutoPtr(clone_());
@@ -1176,8 +1266,8 @@ namespace Action {
         Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(path);
         assert(image.get() != 0);
         image->readMetadata();
-        Exiv2::ExifData& exifData = image->exifData();
-        exifData.setJpegThumbnail(thumbPath);
+        Exiv2::ExifThumb exifThumb(image->exifData());
+        exifThumb.setJpegThumbnail(thumbPath);
         image->writeMetadata();
 
         return 0;
@@ -1209,7 +1299,7 @@ namespace Action {
         assert(image.get() != 0);
         image->readMetadata();
 
-        applyCommands(image.get());
+        int rc = applyCommands(image.get());
 
         // Save both exif and iptc metadata
         image->writeMetadata();
@@ -1217,7 +1307,7 @@ namespace Action {
         if (Params::instance().preserve_) {
             ts.touch(path);
         }
-        return 0;
+        return rc;
     }
     catch(const Exiv2::AnyError& e)
     {
@@ -1227,7 +1317,7 @@ namespace Action {
     }
     } // Modify::run
 
-    void Modify::applyCommands(Exiv2::Image* pImage)
+    int Modify::applyCommands(Exiv2::Image* pImage)
     {
         if (!Params::instance().jpegComment_.empty()) {
             if (Params::instance().verbose_) {
@@ -1243,13 +1333,17 @@ namespace Action {
         ModifyCmds& modifyCmds = Params::instance().modifyCmds_;
         ModifyCmds::const_iterator i = modifyCmds.begin();
         ModifyCmds::const_iterator end = modifyCmds.end();
+        int rc = 0;
+        int ret = 0;
         for (; i != end; ++i) {
             switch (i->cmdId_) {
             case add:
-                addMetadatum(pImage, *i);
+                ret = addMetadatum(pImage, *i);
+                if (rc == 0) rc = ret;
                 break;
             case set:
-                setMetadatum(pImage, *i);
+                ret = setMetadatum(pImage, *i);
+                if (rc == 0) rc = ret;
                 break;
             case del:
                 delMetadatum(pImage, *i);
@@ -1262,9 +1356,10 @@ namespace Action {
                 break;
             }
         }
+        return rc;
     } // Modify::applyCommands
 
-    void Modify::addMetadatum(Exiv2::Image* pImage, const ModifyCmd& modifyCmd)
+    int Modify::addMetadatum(Exiv2::Image* pImage, const ModifyCmd& modifyCmd)
     {
         if (Params::instance().verbose_) {
             std::cout << _("Add") << " " << modifyCmd.key_ << " \""
@@ -1276,7 +1371,8 @@ namespace Action {
         Exiv2::IptcData& iptcData = pImage->iptcData();
         Exiv2::XmpData&  xmpData  = pImage->xmpData();
         Exiv2::Value::AutoPtr value = Exiv2::Value::create(modifyCmd.typeId_);
-        if (0 == value->read(modifyCmd.value_)) {
+        int rc = value->read(modifyCmd.value_);
+        if (0 == rc) {
             if (modifyCmd.metadataId_ == exif) {
                 exifData.add(Exiv2::ExifKey(modifyCmd.key_), value.get());
             }
@@ -1287,11 +1383,19 @@ namespace Action {
                 xmpData.add(Exiv2::XmpKey(modifyCmd.key_), value.get());
             }
         }
+        else {
+            std::cerr << _("Warning") << ": " << modifyCmd.key_ << ": "
+                      << _("Failed to read") << " "
+                      << Exiv2::TypeInfo::typeName(value->typeId())
+                      << " " << _("value")
+                      << " \"" << modifyCmd.value_ << "\"\n";
+        }
+        return rc;
     }
 
     // This function looks rather complex because we try to avoid adding an
     // empty metadatum if reading the value fails
-    void Modify::setMetadatum(Exiv2::Image* pImage, const ModifyCmd& modifyCmd)
+    int Modify::setMetadatum(Exiv2::Image* pImage, const ModifyCmd& modifyCmd)
     {
         if (Params::instance().verbose_) {
             std::cout << _("Set") << " " << modifyCmd.key_ << " \""
@@ -1336,7 +1440,8 @@ namespace Action {
                 && modifyCmd.typeId_ != value->typeId())) {
             value = Exiv2::Value::create(modifyCmd.typeId_);
         }
-        if (0 == value->read(modifyCmd.value_)) {
+        int rc = value->read(modifyCmd.value_);
+        if (0 == rc) {
             if (metadatum) {
                 metadatum->setValue(value.get());
             }
@@ -1355,10 +1460,11 @@ namespace Action {
         else {
             std::cerr << _("Warning") << ": " << modifyCmd.key_ << ": "
                       << _("Failed to read") << " "
-                      << Exiv2::TypeInfo::typeName(value->typeId()) 
+                      << Exiv2::TypeInfo::typeName(value->typeId())
                       << " " << _("value")
                       << " \"" << modifyCmd.value_ << "\"\n";
         }
+        return rc;
     }
 
     void Modify::delMetadatum(Exiv2::Image* pImage, const ModifyCmd& modifyCmd)
@@ -1396,7 +1502,7 @@ namespace Action {
     void Modify::regNamespace(const ModifyCmd& modifyCmd)
     {
         if (Params::instance().verbose_) {
-            std::cout << _("Reg ") << modifyCmd.key_ << "=\"" 
+            std::cout << _("Reg ") << modifyCmd.key_ << "=\""
                       << modifyCmd.value_ << "\"" << std::endl;
         }
         // Registration has been done immediately after parsing the command.
@@ -1601,7 +1707,7 @@ namespace Action {
         // Copy the proprietary tag to the standard place
         if (md != exifData.end()) {
             std::ostringstream os;
-            os << *md;
+            md->write(os, &exifData);
             if (Params::instance().verbose_) {
                 std::cout << _("Setting Exif ISO value to") << " " << os.str() << "\n";
             }
@@ -1637,6 +1743,7 @@ namespace Action {
 // local definitions
 namespace {
 
+    //! @cond IGNORE
     int Timestamp::read(const std::string& path)
     {
         struct stat buf;
@@ -1668,6 +1775,7 @@ namespace {
         buf.modtime = modtime_;
         return utime(path.c_str(), &buf);
     }
+    //! @endcond
 
     int str2Tm(const std::string& timeStr, struct tm* tm)
     {
@@ -1883,11 +1991,11 @@ namespace {
         return 0;
     } // renameFile
 
-    std::string newFilePath(const std::string& path, const std::string& ext) 
+    std::string newFilePath(const std::string& path, const std::string& ext)
     {
         std::string directory = Params::instance().directory_;
         if (directory.empty()) directory = Util::dirname(path);
-        std::string newPath =   directory + EXV_SEPERATOR_STR 
+        std::string newPath =   directory + EXV_SEPERATOR_STR
                               + Util::basename(path, true) + ext;
         return newPath;
     }
