@@ -20,14 +20,14 @@
  */
 /*
   File:      cr2image.cpp
-  Version:   $Rev: 1404 $
+  Version:   $Rev: 1594 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   22-Apr-06, ahu: created
 
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: cr2image.cpp 1404 2008-04-15 03:46:01Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: cr2image.cpp 1594 2008-09-18 11:03:20Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -38,9 +38,10 @@ EXIV2_RCSID("@(#) $Id: cr2image.cpp 1404 2008-04-15 03:46:01Z ahuggel $")
 #endif
 
 #include "cr2image.hpp"
-#include "tiffcomposite.hpp"
-#include "tiffparser.hpp"
-#include "tiffvisitor.hpp"
+#include "cr2image_int.hpp"
+#include "tiffcomposite_int.hpp"
+#include "tiffimage_int.hpp"
+#include "tiffvisitor_int.hpp"
 #include "image.hpp"
 #include "error.hpp"
 #include "futils.hpp"
@@ -56,37 +57,7 @@ EXIV2_RCSID("@(#) $Id: cr2image.cpp 1404 2008-04-15 03:46:01Z ahuggel $")
 // class member definitions
 namespace Exiv2 {
 
-    // CR2 decoder table for special CR2 decoding requirements
-    const TiffDecoderInfo Cr2Decoder::cr2DecoderInfo_[] = {
-        { "*",       Tag::all, Group::ignr,    0 }, // Do not decode tags with group == Group::ignr
-        { "*",         0x014a, Group::ifd0,    0 }, // Todo: Controversial, causes problems with Exiftool
-        { "*",         0x0100, Group::ifd0,    0 }, // CR2 IFD0 refers to a preview image, ignore these tags
-        { "*",         0x0101, Group::ifd0,    0 },
-        { "*",         0x0102, Group::ifd0,    0 },
-        { "*",         0x0103, Group::ifd0,    0 },
-        { "*",         0x0111, Group::ifd0,    0 },
-        { "*",         0x0117, Group::ifd0,    0 },
-        { "*",         0x011a, Group::ifd0,    0 },
-        { "*",         0x011b, Group::ifd0,    0 },
-        { "*",         0x0128, Group::ifd0,    0 },
-        { "*",         0x02bc, Group::ifd0,    &TiffMetadataDecoder::decodeXmp  },
-        { "*",         0x83bb, Group::ifd0,    &TiffMetadataDecoder::decodeIptc },
-        { "*",         0x8649, Group::ifd0,    &TiffMetadataDecoder::decodeIptc }
-    };
-
-    DecoderFct Cr2Decoder::findDecoder(const std::string& make,
-                                             uint32_t     extendedTag,
-                                             uint16_t     group)
-    {
-        DecoderFct decoderFct = &TiffMetadataDecoder::decodeStdTiffEntry;
-        const TiffDecoderInfo* td = find(cr2DecoderInfo_,
-                                         TiffDecoderInfo::Key(make, extendedTag, group));
-        if (td) {
-            // This may set decoderFct to 0, meaning that the tag should not be decoded
-            decoderFct = td->decoderFct_;
-        }
-        return decoderFct;
-    }
+    using namespace Internal;
 
     Cr2Image::Cr2Image(BasicIo::AutoPtr io, bool /*create*/)
         : Image(ImageType::cr2, mdExif | mdIptc, io)
@@ -95,14 +66,20 @@ namespace Exiv2 {
 
     int Cr2Image::pixelWidth() const
     {
-        Exiv2::ExifData::const_iterator widthIter = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
-        return (widthIter == exifData_.end()) ? 0 : widthIter->toLong();
+        ExifData::const_iterator imageWidth = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
+        if (imageWidth != exifData_.end() && imageWidth->count() > 0) {
+            return imageWidth->toLong();
+        }
+        return 0;
     }
 
     int Cr2Image::pixelHeight() const
     {
-        Exiv2::ExifData::const_iterator heightIter = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelYDimension"));
-        return (heightIter == exifData_.end()) ? 0 : heightIter->toLong();
+        ExifData::const_iterator imageHeight = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelYDimension"));
+        if (imageHeight != exifData_.end() && imageHeight->count() > 0) {
+            return imageHeight->toLong();
+        }
+        return 0;
     }
 
     void Cr2Image::setExifData(const ExifData& /*exifData*/)
@@ -138,19 +115,95 @@ namespace Exiv2 {
             throw Error(3, "CR2");
         }
         clearMetadata();
-        Cr2Header cr2Header;
-        TiffParser::decode(this, io_->mmap(), io_->size(),
-                           TiffCreator::create, Cr2Decoder::findDecoder,
-                           &cr2Header);
-
+        ByteOrder bo = Cr2Parser::decode(exifData_,
+                                         iptcData_,
+                                         xmpData_,
+                                         io_->mmap(),
+                                         io_->size());
+        setByteOrder(bo);
     } // Cr2Image::readMetadata
 
     void Cr2Image::writeMetadata()
     {
-        //! Todo: implement me!
+        // Todo: implement me!
         throw(Error(31, "CR2"));
     } // Cr2Image::writeMetadata
 
+    ByteOrder Cr2Parser::decode(
+              ExifData& exifData,
+              IptcData& iptcData,
+              XmpData&  xmpData,
+        const byte*     pData,
+              uint32_t  size
+    )
+    {
+        Cr2Header cr2Header;
+        return TiffParserWorker::decode(exifData,
+                                        iptcData,
+                                        xmpData,
+                                        pData,
+                                        size,
+                                        TiffCreator::create,
+                                        TiffMapping::findDecoder,
+                                        &cr2Header);
+    }
+
+    WriteMethod Cr2Parser::encode(
+              Blob&     blob,
+        const byte*     pData,
+              uint32_t  size,
+        const ExifData& exifData,
+        const IptcData& iptcData,
+        const XmpData&  xmpData
+    )
+    {
+        /* Todo: Implement me!
+
+        TiffParserWorker::encode(blob,
+                                 pData,
+                                 size,
+                                 exifData,
+                                 iptcData,
+                                 xmpData,
+                                 TiffCreator::create,
+                                 TiffMapping::findEncoder);
+        */
+        blob.clear();
+        return wmIntrusive;
+    }
+
+    // *************************************************************************
+    // free functions
+    Image::AutoPtr newCr2Instance(BasicIo::AutoPtr io, bool create)
+    {
+        Image::AutoPtr image(new Cr2Image(io, create));
+        if (!image->good()) {
+            image.reset();
+        }
+        return image;
+    }
+
+    bool isCr2Type(BasicIo& iIo, bool advance)
+    {
+        const int32_t len = 16;
+        byte buf[len];
+        iIo.read(buf, len);
+        if (iIo.error() || iIo.eof()) {
+            return false;
+        }
+        Cr2Header header;
+        bool rc = header.read(buf, len);
+        if (!advance || !rc) {
+            iIo.seek(-len, BasicIo::cur);
+        }
+        return rc;
+    }
+
+}                                       // namespace Exiv2
+
+namespace Exiv2 {
+    namespace Internal {
+
     const char* Cr2Header::cr2sig_ = "CR\2\0";
 
     Cr2Header::Cr2Header()
@@ -184,36 +237,10 @@ namespace Exiv2 {
         return true;
     } // Cr2Header::read
 
-    void Cr2Header::write(Blob& blob) const
+    uint32_t Cr2Header::write(Blob& blob) const
     {
         // Todo: Implement me!
+        return 0;
     }
 
-    // *************************************************************************
-    // free functions
-    Image::AutoPtr newCr2Instance(BasicIo::AutoPtr io, bool create)
-    {
-        Image::AutoPtr image(new Cr2Image(io, create));
-        if (!image->good()) {
-            image.reset();
-        }
-        return image;
-    }
-
-    bool isCr2Type(BasicIo& iIo, bool advance)
-    {
-        const int32_t len = 16;
-        byte buf[len];
-        iIo.read(buf, len);
-        if (iIo.error() || iIo.eof()) {
-            return false;
-        }
-        Cr2Header header;
-        bool rc = header.read(buf, len);
-        if (!advance || !rc) {
-            iIo.seek(-len, BasicIo::cur);
-        }
-        return rc;
-    }
-
-}                                       // namespace Exiv2
+}}                                      // namespace Internal, Exiv2
