@@ -21,7 +21,7 @@
 /*!
   @file    cr2image.hpp
   @brief   Class Cr2Image
-  @version $Rev: 1404 $
+  @version $Rev: 1571 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    22-Apr-06, ahu: created
@@ -33,9 +33,7 @@
 // included header files
 #include "image.hpp"
 #include "basicio.hpp"
-#include "tifffwd.hpp"
 #include "types.hpp"
-#include "tiffimage.hpp"
 
 // + standard includes
 #include <string>
@@ -56,7 +54,7 @@ namespace Exiv2 {
       @brief Class to access raw Canon CR2 images.  Exif metadata
           is supported directly, IPTC is read from the Exif data, if present.
      */
-    class Cr2Image : public Image {
+    class EXIV2API Cr2Image : public Image {
     public:
         //! @name Creators
         //@{
@@ -87,12 +85,12 @@ namespace Exiv2 {
          */
         void writeMetadata();
         /*!
-          @brief Todo: Not supported yet, requires writeMetadata(). Calling 
+          @brief Todo: Not supported yet, requires writeMetadata(). Calling
               this function will throw an Error(32).
          */
         void setExifData(const ExifData& exifData);
         /*!
-          @brief Todo: Not supported yet, requires writeMetadata(). Calling 
+          @brief Todo: Not supported yet, requires writeMetadata(). Calling
               this function will throw an Error(32).
          */
         void setIptcData(const IptcData& iptcData);
@@ -122,60 +120,38 @@ namespace Exiv2 {
     }; // class Cr2Image
 
     /*!
-      @brief Table of Cr2 decoding functions and find function. See
-             TiffDecoder for details.
+      @brief Stateless parser class for data in CR2 format. Images use this
+             class to decode and encode CR2 data.
+             See class TiffParser for details.
      */
-    class Cr2Decoder {
+    class EXIV2API Cr2Parser {
     public:
         /*!
-          @brief Find the decoder function for a key.
+          @brief Decode metadata from a buffer \em pData of length \em size
+                 with data in CR2 format to the provided metadata containers.
+                 See TiffParser::decode().
+        */
+        static ByteOrder decode(
+                  ExifData& exifData,
+                  IptcData& iptcData,
+                  XmpData&  xmpData,
+            const byte*     pData,
+                  uint32_t  size
+        );
+        /*!
+          @brief Encode metadata from the provided metadata to CR2 format.
+                 See TiffParser::encode().
+        */
+        static WriteMethod encode(
+                  Blob&     blob,
+            const byte*     pData,
+                  uint32_t  size,
+            const ExifData& exifData,
+            const IptcData& iptcData,
+            const XmpData&  xmpData
+        );
 
-          If the returned pointer is 0, the tag should not be decoded,
-          else the decoder function should be used.
-
-          @param make Camera make
-          @param extendedTag Extended tag
-          @param group %Group
-
-          @return Pointer to the decoder function
-         */
-        static DecoderFct findDecoder(const std::string& make,
-                                            uint32_t     extendedTag,
-                                            uint16_t     group);
-
-    private:
-        static const TiffDecoderInfo cr2DecoderInfo_[]; //<! CR2 decoder table
-
-    }; // class Cr2Decoder
-
-    /*!
-      @brief Canon CR2 header structure.
-     */
-    class Cr2Header : public TiffHeaderBase {
-    public:
-        //! @name Creators
-        //@{
-        //! Default constructor
-        Cr2Header();
-        //! Destructor.
-        ~Cr2Header();
-        //@}
-
-        //! @name Manipulators
-        //@{
-        bool read(const byte* pData, uint32_t size);
-        //@}
-
-        //! @name Accessors
-        //@{
-        void write(Blob& blob) const;
-        //@}
-
-    private:
-        // DATA
-        uint32_t              offset2_;   //!< Bytes 12-15 from the header
-        static const char*    cr2sig_;    //!< Signature for CR2 type TIFF
-    }; // class Cr2Header
+    }; // class Cr2Parser
 
 // *****************************************************************************
 // template, inline and free functions
@@ -187,10 +163,10 @@ namespace Exiv2 {
              Caller owns the returned object and the auto-pointer ensures that
              it will be deleted.
      */
-    Image::AutoPtr newCr2Instance(BasicIo::AutoPtr io, bool create);
+    EXIV2API Image::AutoPtr newCr2Instance(BasicIo::AutoPtr io, bool create);
 
     //! Check if the file iIo is a CR2 image.
-    bool isCr2Type(BasicIo& iIo, bool advance);
+    EXIV2API bool isCr2Type(BasicIo& iIo, bool advance);
 
 }                                       // namespace Exiv2
 
