@@ -20,14 +20,14 @@
  */
 /*
   File:      crwimage.cpp
-  Version:   $Rev: 1404 $
+  Version:   $Rev: 1704 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   28-Aug-05, ahu: created
 
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: crwimage.cpp 1404 2008-04-15 03:46:01Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: crwimage.cpp 1704 2008-12-16 17:59:07Z ahuggel $")
 
 // Define DEBUG to output debug information to std::cerr, e.g, by calling make
 // like this: make DEFS=-DDEBUG crwimage.o
@@ -42,6 +42,7 @@ EXIV2_RCSID("@(#) $Id: crwimage.cpp 1404 2008-04-15 03:46:01Z ahuggel $")
 #endif
 
 #include "crwimage.hpp"
+#include "crwimage_int.hpp"
 #include "error.hpp"
 #include "futils.hpp"
 #include "value.hpp"
@@ -87,6 +88,165 @@ namespace {
 // class member definitions
 namespace Exiv2 {
 
+    using namespace Internal;
+
+    CrwImage::CrwImage(BasicIo::AutoPtr io, bool /*create*/)
+        : Image(ImageType::crw, mdExif | mdComment, io)
+    {
+    } // CrwImage::CrwImage
+
+    int CrwImage::pixelWidth() const
+    {
+        Exiv2::ExifData::const_iterator widthIter = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
+        if (widthIter != exifData_.end() && widthIter->count() > 0) {
+            return widthIter->toLong();
+        }
+        return 0;
+    }
+
+    int CrwImage::pixelHeight() const
+    {
+        Exiv2::ExifData::const_iterator heightIter = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelYDimension"));
+        if (heightIter != exifData_.end() && heightIter->count() > 0) {
+            return heightIter->toLong();
+        }
+        return 0;
+    }
+
+    void CrwImage::setIptcData(const IptcData& /*iptcData*/)
+    {
+        // not supported
+        throw(Error(32, "IPTC metadata", "CRW"));
+    }
+
+    void CrwImage::readMetadata()
+    {
+#ifdef DEBUG
+        std::cerr << "Reading CRW file " << io_->path() << "\n";
+#endif
+        if (io_->open() != 0) {
+            throw Error(9, io_->path(), strError());
+        }
+        IoCloser closer(*io_);
+        // Ensure that this is the correct image type
+        if (!isCrwType(*io_, false)) {
+            if (io_->error() || io_->eof()) throw Error(14);
+            throw Error(33);
+        }
+        clearMetadata();
+        CrwParser::decode(this, io_->mmap(), io_->size());
+
+    } // CrwImage::readMetadata
+
+    void CrwImage::writeMetadata()
+    {
+#ifdef DEBUG
+        std::cerr << "Writing CRW file " << io_->path() << "\n";
+#endif
+        // Read existing image
+        DataBuf buf;
+        if (io_->open() == 0) {
+            IoCloser closer(*io_);
+            // Ensure that this is the correct image type
+            if (isCrwType(*io_, false)) {
+                // Read the image into a memory buffer
+                buf.alloc(io_->size());
+                io_->read(buf.pData_, buf.size_);
+                if (io_->error() || io_->eof()) {
+                    buf.reset();
+                }
+            }
+        }
+
+        Blob blob;
+        CrwParser::encode(blob, buf.pData_, buf.size_, this);
+
+        // Write new buffer to file
+        BasicIo::AutoPtr tempIo(io_->temporary()); // may throw
+        assert(tempIo.get() != 0);
+        tempIo->write((blob.size() > 0 ? &blob[0] : 0), static_cast<long>(blob.size()));
+        io_->close();
+        io_->transfer(*tempIo); // may throw
+
+    } // CrwImage::writeMetadata
+
+    void CrwParser::decode(CrwImage* pCrwImage, const byte* pData, uint32_t size)
+    {
+        assert(pCrwImage != 0);
+        assert(pData != 0);
+
+        // Parse the image, starting with a CIFF header component
+        CiffHeader::AutoPtr head(new CiffHeader);
+        head->read(pData, size);
+#ifdef DEBUG
+        head->print(std::cerr);
+#endif
+        head->decode(*pCrwImage);
+
+        // a hack to get absolute offset of preview image inside CRW structure
+        CiffComponent* preview = head->findComponent(0x2007, 0x0000);
+        if (preview) {
+            (pCrwImage->exifData())["Exif.Image2.JPEGInterchangeFormat"] = uint32_t(preview->pData() - pData);
+            (pCrwImage->exifData())["Exif.Image2.JPEGInterchangeFormatLength"] = preview->size();
+        }
+    } // CrwParser::decode
+
+    void CrwParser::encode(
+              Blob&     blob,
+        const byte*     pData,
+              uint32_t  size,
+        const CrwImage* pCrwImage
+    )
+    {
+        // Parse image, starting with a CIFF header component
+        CiffHeader::AutoPtr head(new CiffHeader);
+        if (size != 0) {
+            head->read(pData, size);
+        }
+
+        // Encode Exif tags from image into the CRW parse tree and write the
+        // structure to the binary image blob
+        CrwMap::encode(head.get(), *pCrwImage);
+        head->write(blob);
+
+    } // CrwParser::encode
+
+    // *************************************************************************
+    // free functions
+    Image::AutoPtr newCrwInstance(BasicIo::AutoPtr io, bool create)
+    {
+        Image::AutoPtr image(new CrwImage(io, create));
+        if (!image->good()) {
+            image.reset();
+        }
+        return image;
+    }
+
+    bool isCrwType(BasicIo& iIo, bool advance)
+    {
+        bool result = true;
+        byte tmpBuf[14];
+        iIo.read(tmpBuf, 14);
+        if (iIo.error() || iIo.eof()) {
+            return false;
+        }
+        if (!(   ('I' == tmpBuf[0] && 'I' == tmpBuf[1])
+              || ('M' == tmpBuf[0] && 'M' == tmpBuf[1]))) {
+            result = false;
+        }
+        if (   true == result
+            && std::memcmp(tmpBuf + 6, CiffHeader::signature(), 8) != 0) {
+            result = false;
+        }
+        if (!advance || !result) iIo.seek(-14, BasicIo::cur);
+        return result;
+    }
+
+}                                       // namespace Exiv2
+
+namespace Exiv2 {
+    namespace Internal {
+
     /*
       Mapping table used to decode and encode CIFF tags to/from Exif tags.  Only
       a subset of the Exif tags can be mapped to known tags found in CRW files
@@ -165,113 +325,6 @@ namespace Exiv2 {
         { 0xffff, 0xffff }
     };
 
-    CrwImage::CrwImage(BasicIo::AutoPtr io, bool /*create*/)
-        : Image(ImageType::crw, mdExif | mdComment, io)
-    {
-    } // CrwImage::CrwImage
-
-    int CrwImage::pixelWidth() const
-    {
-        Exiv2::ExifData::const_iterator widthIter = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
-        return (widthIter == exifData_.end()) ? 0 : widthIter->toLong();
-    }
-
-    int CrwImage::pixelHeight() const
-    {
-        Exiv2::ExifData::const_iterator heightIter = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelYDimension"));
-        return (heightIter == exifData_.end()) ? 0 : heightIter->toLong();
-    }
-
-    void CrwImage::setIptcData(const IptcData& /*iptcData*/)
-    {
-        // not supported
-        throw(Error(32, "IPTC metadata", "CRW"));
-    }
-
-    void CrwImage::readMetadata()
-    {
-#ifdef DEBUG
-        std::cerr << "Reading CRW file " << io_->path() << "\n";
-#endif
-        if (io_->open() != 0) {
-            throw Error(9, io_->path(), strError());
-        }
-        IoCloser closer(*io_);
-        // Ensure that this is the correct image type
-        if (!isCrwType(*io_, false)) {
-            if (io_->error() || io_->eof()) throw Error(14);
-            throw Error(33);
-        }
-        clearMetadata();
-        CrwParser::decode(this, io_->mmap(), io_->size());
-
-    } // CrwImage::readMetadata
-
-    void CrwImage::writeMetadata()
-    {
-#ifdef DEBUG
-        std::cerr << "Writing CRW file " << io_->path() << "\n";
-#endif
-        // Read existing image
-        DataBuf buf;
-        if (io_->open() == 0) {
-            IoCloser closer(*io_);
-            // Ensure that this is the correct image type
-            if (isCrwType(*io_, false)) {
-                // Read the image into a memory buffer
-                buf.alloc(io_->size());
-                io_->read(buf.pData_, buf.size_);
-                if (io_->error() || io_->eof()) {
-                    buf.reset();
-                }
-            }
-        }
-
-        // Parse image, starting with a CIFF header component
-        CiffHeader::AutoPtr head(new CiffHeader);
-        if (buf.size_ != 0) {
-            head->read(buf.pData_, buf.size_);
-        }
-
-        Blob blob;
-        CrwParser::encode(blob, head.get(), this);
-
-        // Write new buffer to file
-        BasicIo::AutoPtr tempIo(io_->temporary()); // may throw
-        assert (tempIo.get() != 0);
-        tempIo->write(&blob[0], static_cast<long>(blob.size()));
-        io_->close();
-        io_->transfer(*tempIo); // may throw
-
-    } // CrwImage::writeMetadata
-
-    void CrwParser::decode(CrwImage* pCrwImage, const byte* pData, uint32_t size)
-    {
-        assert(pCrwImage != 0);
-        assert(pData != 0);
-
-        // Parse the image, starting with a CIFF header component
-        CiffHeader::AutoPtr head(new CiffHeader);
-        head->read(pData, size);
-#ifdef DEBUG
-        head->print(std::cerr);
-#endif
-        head->decode(*pCrwImage);
-
-    } // CrwParser::decode
-
-    void CrwParser::encode(Blob& blob, CiffHeader* pHead, const CrwImage* pCrwImage)
-    {
-        assert(pCrwImage != 0);
-        assert(pHead != 0);
-
-        // Encode Exif tags from image into the Crw parse tree and write the structure
-        // to the binary image blob
-        CrwMap::encode(pHead, *pCrwImage);
-        pHead->write(blob);
-
-    } // CrwParser::encode
-
     const char CiffHeader::signature_[] = "HEAPCCDR";
 
     CiffHeader::~CiffHeader()
@@ -1046,8 +1099,8 @@ namespace Exiv2 {
                                     Image&         image,
                                     ByteOrder      /*byteOrder*/)
     {
-        image.exifData().setJpegThumbnail(ciffComponent.pData(),
-                                          ciffComponent.size());
+        ExifThumb exifThumb(image.exifData());
+        exifThumb.setJpegThumbnail(ciffComponent.pData(), ciffComponent.size());
     } // CrwMap::decode0x2008
 
     void CrwMap::decodeBasic(const CiffComponent& ciffComponent,
@@ -1269,7 +1322,7 @@ namespace Exiv2 {
                 edY->copy(buf.pData_ + 4, pHead->byteOrder());
             }
             int32_t d = 0;
-            if (edO != edEnd && edO->typeId() == unsignedShort) {
+            if (edO != edEnd && edO->count() > 0 && edO->typeId() == unsignedShort) {
                 d = RotationMap::degrees(static_cast<uint16_t>(edO->toLong()));
             }
             l2Data(buf.pData_ + 12, d, pHead->byteOrder());
@@ -1287,7 +1340,8 @@ namespace Exiv2 {
         assert(pCrwMapping != 0);
         assert(pHead != 0);
 
-        DataBuf buf = image.exifData().copyThumbnail();
+        ExifThumbC exifThumb(image.exifData());
+        DataBuf buf = exifThumb.copy();
         if (buf.size_ != 0) {
             pHead->add(pCrwMapping->crwTagId_, pCrwMapping->crwDir_, buf);
         }
@@ -1298,35 +1352,6 @@ namespace Exiv2 {
 
     // *************************************************************************
     // free functions
-    Image::AutoPtr newCrwInstance(BasicIo::AutoPtr io, bool create)
-    {
-        Image::AutoPtr image(new CrwImage(io, create));
-        if (!image->good()) {
-            image.reset();
-        }
-        return image;
-    }
-
-    bool isCrwType(BasicIo& iIo, bool advance)
-    {
-        bool result = true;
-        byte tmpBuf[14];
-        iIo.read(tmpBuf, 14);
-        if (iIo.error() || iIo.eof()) {
-            return false;
-        }
-        if (!(   ('I' == tmpBuf[0] && 'I' == tmpBuf[1])
-              || ('M' == tmpBuf[0] && 'M' == tmpBuf[1]))) {
-            result = false;
-        }
-        if (   true == result
-            && std::memcmp(tmpBuf + 6, CiffHeader::signature(), 8) != 0) {
-            result = false;
-        }
-        if (!advance || !result) iIo.seek(-14, BasicIo::cur);
-        return result;
-    }
-
     DataBuf packIfdId(const ExifData& exifData,
                             IfdId     ifdId,
                             ByteOrder byteOrder)
@@ -1350,11 +1375,12 @@ namespace Exiv2 {
         return buf;
     }
 
-}                                       // namespace Exiv2
+}}                                       // namespace Internal, Exiv2
 
 // *****************************************************************************
 // local definitions
 namespace {
+    //! @cond IGNORE
     const RotationMap::OmList RotationMap::omList_[] = {
         { 1,    0 },
         { 3,  180 },
@@ -1390,4 +1416,5 @@ namespace {
         }
         return d;
     }
+    //! @endcond
 }
