@@ -20,14 +20,14 @@
  */
 /*
   File:      datasets.cpp
-  Version:   $Rev: 1366 $
+  Version:   $Rev: 1614 $
   Author(s): Brad Schick (brad) <brad@robotbattle.com>
              Gilles Caulier (gc) <caulier.gilles@kdemail.net>
   History:   24-Jul-04, brad: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: datasets.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: datasets.cpp 1614 2008-09-27 03:11:05Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -608,6 +608,10 @@ namespace Exiv2 {
     {
     }
 
+    IptcKey::~IptcKey()
+    {
+    }
+
     IptcKey& IptcKey::operator=(const IptcKey& rhs)
     {
         if (this == &rhs) return *this;
@@ -634,7 +638,7 @@ namespace Exiv2 {
         std::string::size_type pos1 = key_.find('.');
         if (pos1 == std::string::npos) throw Error(6, key_);
         std::string familyName = key_.substr(0, pos1);
-        if (familyName != std::string(familyName_)) {
+        if (0 != strcmp(familyName.c_str(), familyName_)) {
             throw Error(6, key_);
         }
         std::string::size_type pos0 = pos1 + 1;
