@@ -20,7 +20,7 @@
  */
 /*
   File:      fujimn.cpp
-  Version:   $Rev: 1473 $
+  Version:   $Rev: 1512 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Gilles Caulier (gc) <caulier.gilles@kdemail.net>
   History:   18-Feb-04, ahu: created
@@ -29,13 +29,12 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: fujimn.cpp 1473 2008-06-03 02:15:04Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: fujimn.cpp 1512 2008-06-21 10:28:17Z ahuggel $")
 
 // *****************************************************************************
 // included header files
 #include "types.hpp"
 #include "fujimn.hpp"
-#include "makernote.hpp"
 #include "value.hpp"
 #include "i18n.h"                // NLS support.
 
@@ -50,15 +49,6 @@ EXIV2_RCSID("@(#) $Id: fujimn.cpp 1473 2008-06-03 02:15:04Z ahuggel $")
 // class member definitions
 namespace Exiv2 {
 
-    //! @cond IGNORE
-    FujiMakerNote::RegisterMn::RegisterMn()
-    {
-        MakerNoteFactory::registerMakerNote("FUJIFILM", "*", createFujiMakerNote);
-        MakerNoteFactory::registerMakerNote(
-            fujiIfdId, MakerNote::AutoPtr(new FujiMakerNote));
-    }
-    //! @endcond
-
     //! OffOn, multiple tags
     extern const TagDetails fujiOffOn[] = {
         { 0, N_("Off") },
@@ -293,81 +283,4 @@ namespace Exiv2 {
         return tagInfo_;
     }
 
-    FujiMakerNote::FujiMakerNote(bool alloc)
-        : IfdMakerNote(fujiIfdId, alloc)
-    {
-        byteOrder_ = littleEndian;
-        absShift_ = false;
-        byte buf[] = {
-            'F', 'U', 'J', 'I', 'F', 'I', 'L', 'M', 0x0c, 0x00, 0x00, 0x00
-        };
-        readHeader(buf, 12, byteOrder_);
-    }
-
-    FujiMakerNote::FujiMakerNote(const FujiMakerNote& rhs)
-        : IfdMakerNote(rhs)
-    {
-    }
-
-    int FujiMakerNote::readHeader(const byte* buf,
-                                  long len,
-                                  ByteOrder /*byteOrder*/)
-    {
-        if (len < 12) return 1;
-
-        header_.alloc(12);
-        std::memcpy(header_.pData_, buf, header_.size_);
-        // Read offset to the IFD relative to the start of the makernote
-        // from the header. Note that we ignore the byteOrder paramter
-        start_ = getUShort(header_.pData_ + 8, byteOrder_);
-        return 0;
-    }
-
-    int FujiMakerNote::checkHeader() const
-    {
-        int rc = 0;
-        // Check the FUJIFILM prefix
-        if (   header_.size_ < 12
-            || std::string(reinterpret_cast<char*>(header_.pData_), 8)
-                    != std::string("FUJIFILM", 8)) {
-            rc = 2;
-        }
-        return rc;
-    }
-
-    FujiMakerNote::AutoPtr FujiMakerNote::create(bool alloc) const
-    {
-        return AutoPtr(create_(alloc));
-    }
-
-    FujiMakerNote* FujiMakerNote::create_(bool alloc) const
-    {
-        AutoPtr makerNote(new FujiMakerNote(alloc));
-        assert(makerNote.get() != 0);
-        makerNote->readHeader(header_.pData_, header_.size_, byteOrder_);
-        return makerNote.release();
-    }
-
-    FujiMakerNote::AutoPtr FujiMakerNote::clone() const
-    {
-        return AutoPtr(clone_());
-    }
-
-    FujiMakerNote* FujiMakerNote::clone_() const
-    {
-        return new FujiMakerNote(*this);
-    }
-
-// *****************************************************************************
-// free functions
-
-    MakerNote::AutoPtr createFujiMakerNote(bool        alloc,
-                                           const byte* /*buf*/,
-                                           long        /*len*/,
-                                           ByteOrder   /*byteOrder*/,
-                                           long        /*offset*/)
-    {
-        return MakerNote::AutoPtr(new FujiMakerNote(alloc));
-    }
-
 }                                       // namespace Exiv2
