@@ -21,7 +21,7 @@
 /*
   File:      i18n.h
   Brief:     i18n definitions. Do not use. This is an Exiv2 internal header.
-  Version:   $Rev$
+  Version:   $Rev: 1703 $
   Author(s): Gilles Caulier (gc) <caulier.gilles@kdemail.net>
   History:   01-Nov-06, gc: created
  */
@@ -39,7 +39,7 @@
 # include <libintl.h>
 
 // Definition is in types.cpp
-const char* _exvGettext(const char* str);
+EXIV2API const char* _exvGettext(const char* str);
 
 # define _(String) _exvGettext(String)
 # define N_(String) String
