@@ -20,8 +20,8 @@
  */
 /*!
   @file    image.hpp
-  @brief   Class JpegImage to access JPEG images
-  @version $Rev: 1404 $
+  @brief   Class Image, defining the interface for all Image subclasses.
+  @version $Rev: 1648 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @author  Brad Schick (brad)
@@ -50,12 +50,6 @@
 // namespace extensions
 namespace Exiv2 {
 
-// *****************************************************************************
-// type definitions
-
-    //! Container for binary image
-    typedef std::vector<byte> Blob;
-
 // *****************************************************************************
 // class definitions
 
@@ -74,7 +68,7 @@ namespace Exiv2 {
       ImageFactory method. The Image class can then be used to to read, write,
       and save metadata.
      */
-    class Image {
+    class EXIV2API Image {
     public:
         //! Image auto_ptr type
         typedef std::auto_ptr<Image> AutoPtr;
@@ -83,10 +77,10 @@ namespace Exiv2 {
         //@{
         /*!
           @brief Constructor taking the image type, a bitmap of the supported
-              metadata types and an auto-pointer that owns an IO instance. 
+              metadata types and an auto-pointer that owns an IO instance.
               See subclass constructor doc.
          */
-        Image(int              imageType, 
+        Image(int              imageType,
               uint16_t         supportedMetadata,
               BasicIo::AutoPtr io);
         //! Virtual Destructor
@@ -148,8 +142,8 @@ namespace Exiv2 {
           @brief Assign a raw XMP packet. The new XMP packet is not written
               to the image until the writeMetadata() method is called.
 
-          Subsequent calls to writeMetadata() write the XMP packet from 
-          the buffered raw XMP packet rather than from buffered parsed XMP 
+          Subsequent calls to writeMetadata() write the XMP packet from
+          the buffered raw XMP packet rather than from buffered parsed XMP
           data. In order to write from parsed XMP data again, use
           either writeXmpFromPacket(false) or setXmpData().
 
@@ -160,10 +154,10 @@ namespace Exiv2 {
           @brief Erase the buffered XMP packet. XMP data is not removed from
               the actual image until the writeMetadata() method is called.
 
-          This has the same effect as clearXmpData() but operates on the 
+          This has the same effect as clearXmpData() but operates on the
           buffered raw XMP packet only, not the parsed XMP data.
 
-          Subsequent calls to writeMetadata() write the XMP packet from 
+          Subsequent calls to writeMetadata() write the XMP packet from
           the buffered raw XMP packet rather than from buffered parsed XMP
           data. In order to write from parsed XMP data again, use
           either writeXmpFromPacket(false) or setXmpData().
@@ -175,7 +169,7 @@ namespace Exiv2 {
 
           Subsequent calls to writeMetadata() encode the XMP data to
           a raw XMP packet and write the newly encoded packet to the image.
-          In the process, the buffered raw XMP packet is updated. 
+          In the process, the buffered raw XMP packet is updated.
           In order to write directly from the raw XMP packet, use
           writeXmpFromPacket(true) or setXmpPacket().
 
@@ -186,7 +180,7 @@ namespace Exiv2 {
           @brief Erase any buffered XMP data. XMP data is not removed from
               the actual image until the writeMetadata() method is called.
 
-          This has the same effect as clearXmpPacket() but operates on the 
+          This has the same effect as clearXmpPacket() but operates on the
           buffered parsed XMP data.
 
           Subsequent calls to writeMetadata() encode the XMP data to
@@ -266,18 +260,33 @@ namespace Exiv2 {
           XMP from the buffered raw XMP packet or from parsed XMP data.
           The default is to write from parsed XMP data. The switch is also
           set by all functions to set and clear the buffered raw XMP packet
-          and parsed XMP data, so using this function should usually not be 
+          and parsed XMP data, so using this function should usually not be
           necessary.
 
-          If %Exiv2 was compiled without XMP support, the default for this 
-          flag is true and it will never be changed in order to preserve 
+          If %Exiv2 was compiled without XMP support, the default for this
+          flag is true and it will never be changed in order to preserve
           access to the raw XMP packet.
          */
         void writeXmpFromPacket(bool flag);
+        /*!
+          @brief Set the byte order to encode the Exif metadata in.
+
+          The setting is only used when new Exif metadata is created and may
+          not be applicable at all for some image formats. If the target image
+          already contains Exif metadata, the byte order of the existing data
+          is used. If byte order is not set when writeMetadata() is called,
+          little-endian byte order (II) is used by default.
+         */
+        void setByteOrder(ByteOrder byteOrder);
         //@}
 
         //! @name Accessors
         //@{
+        /*!
+          @brief Return the byte order in which the Exif metadata of the image is
+                 encoded. Initially, it is not set (\em invalidByteOrder).
+         */
+        ByteOrder byteOrder() const { return byteOrder_; }
         /*!
           @brief Check if the Image instance is valid. Use after object
               construction.
@@ -402,6 +411,7 @@ namespace Exiv2 {
         const int         imageType_;         //!< Image type
         const uint16_t    supportedMetadata_; //!< Bitmap with all supported metadata types
         bool              writeXmpFromPacket_;//!< Determines the source when writing XMP
+        ByteOrder         byteOrder_;         //!< Byte order
 
     }; // class Image
 
@@ -415,7 +425,7 @@ namespace Exiv2 {
 
       The factory is implemented as a static class.
     */
-    class ImageFactory {
+    class EXIV2API ImageFactory {
         friend bool Image::good() const;
     public:
         /*!
@@ -549,21 +559,6 @@ namespace Exiv2 {
         static bool checkType(int type, BasicIo& io, bool advance);
 
     private:
-        //! Struct for storing image types and function pointers.
-        struct Registry {
-            //! Comparison operator to compare a Registry structure with an image type
-            bool operator==(const int& imageType) const;
-
-            // DATA
-            int            imageType_;
-            NewInstanceFct newInstance_;
-            IsThisTypeFct  isThisType_;
-            AccessMode     exifSupport_;
-            AccessMode     iptcSupport_;
-            AccessMode     xmpSupport_;
-            AccessMode     commentSupport_;
-        };
-
         //! @name Creators
         //@{
         //! Prevent construction: not implemented.
@@ -572,65 +567,8 @@ namespace Exiv2 {
         ImageFactory(const ImageFactory& rhs);
         //@}
 
-        // DATA
-        //! List of image types, creation functions and access modes
-        static const Registry registry_[];
     }; // class ImageFactory
 
-    //! Helper class modelling the TIFF header structure.
-    class TiffHeader {
-    public:
-        //! @name Creators
-        //@{
-        /*!
-          @brief Default constructor. Optionally sets the byte order
-                 (default: little endian).
-         */
-        explicit TiffHeader(ByteOrder byteOrder =littleEndian);
-        //@}
-
-        //! @name Manipulators
-        //@{
-        //! Read the TIFF header from a data buffer. Returns 0 if successful.
-        int read(const byte* buf);
-        //@}
-
-        //! @name Accessors
-        //@{
-        /*!
-          @brief Write a standard TIFF header into buf as a data string, return
-                 number of bytes copied.
-
-          Only the byte order of the TIFF header varies, the values written for
-          offset and tag are constant, i.e., independent of the values possibly
-          read before a call to this function. The value 0x00000008 is written
-          for the offset, tag is set to 0x002a.
-
-          @param buf The data buffer to write to.
-          @return The number of bytes written.
-         */
-        long copy(byte* buf) const;
-        //! Return the size of the TIFF header in bytes.
-        long size() const { return 8; }
-        //! Return the byte order (little or big endian).
-        ByteOrder byteOrder() const { return byteOrder_; }
-        //! Return the tag value.
-        uint16_t tag() const { return tag_; }
-        /*!
-          @brief Return the offset to IFD0 from the start of the TIFF header.
-                 The offset is 0x00000008 if IFD0 begins immediately after the
-                 TIFF header.
-         */
-        uint32_t offset() const { return offset_; }
-        //@}
-
-    private:
-        ByteOrder byteOrder_;
-        uint16_t tag_;
-        uint32_t offset_;
-
-    }; // class TiffHeader
-
 // *****************************************************************************
 // template, inline and free functions
 
