@@ -20,7 +20,7 @@
  */
 /*
   File:      jpgimage.cpp
-  Version:   $Rev: 1503 $
+  Version:   $Rev: 1704 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Brad Schick (brad) <brad@robotbattle.com>
   History:   15-Jan-05, brad: split out from image.cpp
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: jpgimage.cpp 1503 2008-06-14 13:41:48Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: jpgimage.cpp 1704 2008-12-16 17:59:07Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -137,7 +137,7 @@ namespace Exiv2 {
                 return -2;
             }
 #ifndef DEBUG
-            if (   (dataSize & 1) 
+            if (   (dataSize & 1)
                 && position + dataSize == static_cast<uint32_t>(sizePsData)) {
                 std::cerr << "Warning: "
                           << "Photoshop IRB data is not padded to even size\n";
@@ -197,7 +197,7 @@ namespace Exiv2 {
             append(psBlob, pPsData, sizeFront);
         }
         // Write new iptc record if we have it
-        DataBuf rawIptc(iptcData.copy());
+        DataBuf rawIptc = IptcParser::encode(iptcData);
         if (rawIptc.size_ > 0) {
             byte tmpBuf[12];
             std::memcpy(tmpBuf, Photoshop::bimId_, 4);
@@ -212,7 +212,7 @@ namespace Exiv2 {
         }
         // Write existing stuff after record, data is rounded to be even.
         const uint32_t sizeOldData = sizeHdr + sizeIptc + (sizeIptc & 1);
-        // Note: Because of the rounding, sizeFront + sizeOldData can be 
+        // Note: Because of the rounding, sizeFront + sizeOldData can be
         // _greater_ than sizePsData by 1 (not just equal), if the original
         // data was not padded.
         if (static_cast<uint32_t>(sizePsData) > sizeFront + sizeOldData) {
@@ -283,6 +283,7 @@ namespace Exiv2 {
         DataBuf buf(bufMinSize);
         Blob iptcBlob;
         bool foundPsData = false;
+        bool foundExifData = false;
 
         // Read section marker
         int marker = advanceToMarker();
@@ -305,7 +306,8 @@ namespace Exiv2 {
                 if (--search == 0) break;
             }
 
-            if (marker == app1_ && memcmp(buf.pData_ + 2, exifId_, 6) == 0) {
+            if (   !foundExifData
+                && marker == app1_ && memcmp(buf.pData_ + 2, exifId_, 6) == 0) {
                 if (size < 8) {
                     rc = 1;
                     break;
@@ -315,13 +317,16 @@ namespace Exiv2 {
                 DataBuf rawExif(size - 8);
                 io_->read(rawExif.pData_, rawExif.size_);
                 if (io_->error() || io_->eof()) throw Error(14);
-                if (exifData_.load(rawExif.pData_, rawExif.size_)) {
+                ByteOrder bo = ExifParser::decode(exifData_, rawExif.pData_, rawExif.size_);
+                setByteOrder(bo);
+                if (rawExif.size_ > 0 && byteOrder() == invalidByteOrder) {
 #ifndef SUPPRESS_WARNINGS
                     std::cerr << "Warning: Failed to decode Exif metadata.\n";
 #endif
                     exifData_.clear();
                 }
                 --search;
+                foundExifData = true;
             }
             else if (marker == app1_ && memcmp(buf.pData_ + 2, xmpId_, 29) == 0) {
                 if (size < 31) {
@@ -401,7 +406,7 @@ namespace Exiv2 {
             }
             else if (   pixelHeight_ == 0
                      && (   marker == sof0_  || marker == sof1_  || marker == sof2_
-                         || marker == sof3_  || marker == sof5_  || marker == sof6_ 
+                         || marker == sof3_  || marker == sof5_  || marker == sof6_
                          || marker == sof7_  || marker == sof9_  || marker == sof10_
                          || marker == sof11_ || marker == sof13_ || marker == sof14_
                          || marker == sof15_)) {
@@ -422,7 +427,7 @@ namespace Exiv2 {
                     break;
                 }
                 // Skip the remainder of the unknown segment
-                if (io_->seek(size-bufRead, BasicIo::cur)) throw Error(14);
+                if (io_->seek(size - bufRead, BasicIo::cur)) throw Error(14);
             }
             // Read the beginning of the next segment
             marker = advanceToMarker();
@@ -433,7 +438,9 @@ namespace Exiv2 {
         } // while there are segments to process
 
         if (   iptcBlob.size() > 0
-            && iptcData_.load(&iptcBlob[0], static_cast<long>(iptcBlob.size()))) {
+            && IptcParser::decode(iptcData_,
+                                  &iptcBlob[0],
+                                  static_cast<uint32_t>(iptcBlob.size()))) {
 #ifndef SUPPRESS_WARNINGS
             std::cerr << "Warning: Failed to decode IPTC metadata.\n";
 #endif
@@ -484,6 +491,7 @@ namespace Exiv2 {
         int skipApp13Ps3 = -1;
         int skipCom = -1;
         DataBuf psData;
+        DataBuf rawExif;
 
         // Write image header
         if (writeHeader(outIo)) throw Error(21);
@@ -506,11 +514,16 @@ namespace Exiv2 {
                 insertPos = count + 1;
                 if (io_->seek(size-bufRead, BasicIo::cur)) throw Error(22);
             }
-            else if (marker == app1_ && memcmp(buf.pData_ + 2, exifId_, 6) == 0) {
+            else if (   skipApp1Exif == -1
+                     && marker == app1_ && memcmp(buf.pData_ + 2, exifId_, 6) == 0) {
                 if (size < 8) throw Error(22);
                 skipApp1Exif = count;
                 ++search;
-                if (io_->seek(size-bufRead, BasicIo::cur)) throw Error(22);
+                // Seek to beginning and read the current Exif data
+                io_->seek(8 - bufRead, BasicIo::cur);
+                rawExif.alloc(size - 8);
+                io_->read(rawExif.pData_, rawExif.size_);
+                if (io_->error() || io_->eof()) throw Error(22);
             }
             else if (marker == app1_ && memcmp(buf.pData_ + 2, xmpId_, 29) == 0) {
                 if (size < 31) throw Error(22);
@@ -589,26 +602,42 @@ namespace Exiv2 {
                     --search;
                 }
                 if (exifData_.count() > 0) {
-                    DataBuf rawExif = exifData_.copy();
-                    if (rawExif.size_ > 0) {
+                    Blob blob;
+                    ByteOrder bo = byteOrder();
+                    if (bo == invalidByteOrder) {
+                        bo = littleEndian;
+                        setByteOrder(bo);
+                    }
+                    WriteMethod wm = ExifParser::encode(blob,
+                                                        rawExif.pData_,
+                                                        rawExif.size_,
+                                                        bo,
+                                                        exifData_);
+                    const byte* pExifData = rawExif.pData_;
+                    uint32_t exifSize = rawExif.size_;
+                    if (wm == wmIntrusive) {
+                        pExifData = blob.size() > 0 ? &blob[0] : 0;
+                        exifSize = static_cast<uint32_t>(blob.size());
+                    }
+                    if (exifSize > 0) {
                         // Write APP1 marker, size of APP1 field, Exif id and Exif data
                         tmpBuf[0] = 0xff;
                         tmpBuf[1] = app1_;
 
-                        if (rawExif.size_ + 8 > 0xffff) throw Error(37, "Exif");
-                        us2Data(tmpBuf + 2, static_cast<uint16_t>(rawExif.size_ + 8), bigEndian);
+                        if (exifSize + 8 > 0xffff) throw Error(37, "Exif");
+                        us2Data(tmpBuf + 2, static_cast<uint16_t>(exifSize + 8), bigEndian);
                         std::memcpy(tmpBuf + 4, exifId_, 6);
                         if (outIo.write(tmpBuf, 10) != 10) throw Error(21);
 
                         // Write new Exif data buffer
-                        if (   outIo.write(rawExif.pData_, rawExif.size_)
-                            != rawExif.size_) throw Error(21);
+                        if (   outIo.write(pExifData, exifSize)
+                            != static_cast<long>(exifSize)) throw Error(21);
                         if (outIo.error()) throw Error(21);
                         --search;
                     }
                 }
                 if (writeXmpFromPacket() == false) {
-                    if (XmpParser::encode(xmpPacket_, xmpData_)) {
+                    if (XmpParser::encode(xmpPacket_, xmpData_) > 1) {
 #ifndef SUPPRESS_WARNINGS
                         std::cerr << "Error: Failed to encode XMP metadata.\n";
 #endif
