@@ -20,7 +20,7 @@
  */
 /*
   File:      metadatum.cpp
-  Version:   $Rev: 1391 $
+  Version:   $Rev: 1668 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Brad Schick (brad) <brad@robotbattle.com>
   History:   26-Jan-04, ahu: created
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: metadatum.cpp 1391 2008-02-16 18:08:44Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: metadatum.cpp 1668 2008-11-05 15:23:49Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -43,6 +43,10 @@ EXIV2_RCSID("@(#) $Id: metadatum.cpp 1391 2008-02-16 18:08:44Z ahuggel $")
 // class member definitions
 namespace Exiv2 {
 
+    Key::~Key()
+    {
+    }
+
     Key::AutoPtr Key::clone() const
     {
         return AutoPtr(clone_());
@@ -70,10 +74,10 @@ namespace Exiv2 {
         return *this;
     }
 
-    std::string Metadatum::print() const
+    std::string Metadatum::print(const ExifData* pMetadata) const
     {
         std::ostringstream os;
-        write(os);
+        write(os, pMetadata);
         return os.str();
     }
 
