@@ -20,14 +20,14 @@
  */
 /*
   File:      mrwimage.cpp
-  Version:   $Rev: 1404 $
+  Version:   $Rev: 1572 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   13-May-06, ahu: created
   Credits:   See header file
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: mrwimage.cpp 1404 2008-04-15 03:46:01Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: mrwimage.cpp 1572 2008-09-05 14:49:44Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -38,7 +38,7 @@ EXIV2_RCSID("@(#) $Id: mrwimage.cpp 1404 2008-04-15 03:46:01Z ahuggel $")
 #endif
 
 #include "mrwimage.hpp"
-#include "tiffparser.hpp"
+#include "tiffimage.hpp"
 #include "image.hpp"
 #include "basicio.hpp"
 #include "error.hpp"
@@ -61,9 +61,8 @@ namespace Exiv2 {
 
     int MrwImage::pixelWidth() const
     {
-        ExifData::const_iterator imageWidth;
-        if ((imageWidth = exifData_.findKey(Exiv2::ExifKey("Exif.Image.ImageWidth"))) != exifData_.end())
-        {
+        ExifData::const_iterator imageWidth = exifData_.findKey(Exiv2::ExifKey("Exif.Image.ImageWidth"));
+        if (imageWidth != exifData_.end() && imageWidth->count() > 0) {
             return imageWidth->toLong();
         }
         return 0;
@@ -71,9 +70,8 @@ namespace Exiv2 {
 
     int MrwImage::pixelHeight() const
     {
-        ExifData::const_iterator imageHeight;
-        if ((imageHeight = exifData_.findKey(Exiv2::ExifKey("Exif.Image.ImageLength"))) != exifData_.end())
-        {
+        ExifData::const_iterator imageHeight = exifData_.findKey(Exiv2::ExifKey("Exif.Image.ImageLength"));
+        if (imageHeight != exifData_.end() && imageHeight->count() > 0) {
             return imageHeight->toLong();
         }
         return 0;
@@ -142,8 +140,12 @@ namespace Exiv2 {
         io_->read(buf.pData_, buf.size_);
         if (io_->error() || io_->eof()) throw Error(14);
 
-        TiffParser::decode(this, buf.pData_, buf.size_,
-                           TiffCreator::create, TiffDecoder::findDecoder);
+        ByteOrder bo = TiffParser::decode(exifData_,
+                                          iptcData_,
+                                          xmpData_,
+                                          buf.pData_,
+                                          buf.size_);
+        setByteOrder(bo);
     } // MrwImage::readMetadata
 
     void MrwImage::writeMetadata()
