@@ -21,7 +21,7 @@
 /*!
   @file    orfimage.hpp
   @brief   Olympus RAW image
-  @version $Rev: 1404 $
+  @version $Rev: 1703 $
   @author  Jeff Costlow
            <a href="mailto:costlow@gmail.com">costlow@gmail.com</a>
   @date    31-Jul-07, costlow: created
@@ -33,7 +33,7 @@
 // included header files
 #include "image.hpp"
 #include "basicio.hpp"
-#include "tiffimage.hpp"
+#include "types.hpp"
 
 // + standard includes
 #include <string>
@@ -54,7 +54,7 @@ namespace Exiv2 {
       @brief Class to access raw Olympus ORF images. Exif metadata is supported
              directly, IPTC is read from the Exif data, if present.
      */
-    class OrfImage : public Image {
+    class EXIV2API OrfImage : public Image {
     public:
         //! @name Creators
         //@{
@@ -85,12 +85,12 @@ namespace Exiv2 {
          */
         void writeMetadata();
         /*!
-          @brief Todo: Not supported yet, requires writeMetadata(). Calling 
+          @brief Todo: Not supported yet, requires writeMetadata(). Calling
               this function will throw an Error(32).
          */
         void setExifData(const ExifData& exifData);
         /*!
-          @brief Todo: Not supported yet, requires writeMetadata(). Calling 
+          @brief Todo: Not supported yet, requires writeMetadata(). Calling
               this function will throw an Error(32).
          */
         void setIptcData(const IptcData& iptcData);
@@ -120,28 +120,38 @@ namespace Exiv2 {
     }; // class OrfImage
 
     /*!
-      @brief Olympus ORF header structure.
+      @brief Stateless parser class for data in ORF format. Images use this
+             class to decode and encode ORF data.
+             See class TiffParser for details.
      */
-    class OrfHeader : public TiffHeaderBase {
+    class EXIV2API OrfParser {
     public:
-        //! @name Creators
-        //@{
-        //! Default constructor
-        OrfHeader();
-        //! Destructor.
-        ~OrfHeader();
-        //@}
+        /*!
+          @brief Decode metadata from a buffer \em pData of length \em size
+                 with data in ORF format to the provided metadata containers.
+                 See TiffParser::decode().
+        */
+        static ByteOrder decode(
+                  ExifData& exifData,
+                  IptcData& iptcData,
+                  XmpData&  xmpData,
+            const byte*     pData,
+                  uint32_t  size
+        );
+        /*!
+          @brief Encode metadata from the provided metadata to ORF format.
+                 See TiffParser::encode().
+        */
+        static WriteMethod encode(
+                  Blob&     blob,
+            const byte*     pData,
+                  uint32_t  size,
+            const ExifData& exifData,
+            const IptcData& iptcData,
+            const XmpData&  xmpData
+        );
 
-        //! @name Manipulators
-        //@{
-        bool read(const byte* pData, uint32_t size);
-        //@}
-
-        //! @name Accessors
-        //@{
-        void write(Blob& blob) const;
-        //@}
-    }; // class OrfHeader
+    }; // class OrfParser
 
 // *****************************************************************************
 // template, inline and free functions
@@ -153,10 +163,10 @@ namespace Exiv2 {
              Caller owns the returned object and the auto-pointer ensures that
              it will be deleted.
      */
-    Image::AutoPtr newOrfInstance(BasicIo::AutoPtr io, bool create);
+    EXIV2API Image::AutoPtr newOrfInstance(BasicIo::AutoPtr io, bool create);
 
     //! Check if the file iIo is an ORF image.
-    bool isOrfType(BasicIo& iIo, bool advance);
+    EXIV2API bool isOrfType(BasicIo& iIo, bool advance);
 
 }                                       // namespace Exiv2
 
