@@ -20,14 +20,14 @@
  */
 /*
   File:      rafimage.cpp
-  Version:   $Rev: 1404 $
+  Version:   $Rev: 1620 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   05-Feb-07, ahu: created
   Credits:   See header file
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: rafimage.cpp 1404 2008-04-15 03:46:01Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: rafimage.cpp 1620 2008-10-09 14:10:51Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -38,7 +38,7 @@ EXIV2_RCSID("@(#) $Id: rafimage.cpp 1404 2008-04-15 03:46:01Z ahuggel $")
 #endif
 
 #include "rafimage.hpp"
-#include "tiffparser.hpp"
+#include "tiffimage.hpp"
 #include "image.hpp"
 #include "basicio.hpp"
 #include "error.hpp"
@@ -62,13 +62,19 @@ namespace Exiv2 {
     int RafImage::pixelWidth() const
     {
         Exiv2::ExifData::const_iterator widthIter = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelXDimension"));
-        return (widthIter == exifData_.end()) ? 0 : widthIter->toLong();
+        if (widthIter != exifData_.end() && widthIter->count() > 0) {
+            return widthIter->toLong();
+        }
+        return 0;
     }
 
     int RafImage::pixelHeight() const
     {
         Exiv2::ExifData::const_iterator heightIter = exifData_.findKey(Exiv2::ExifKey("Exif.Photo.PixelYDimension"));
-        return (heightIter == exifData_.end()) ? 0 : heightIter->toLong();
+        if (heightIter != exifData_.end() && heightIter->count() > 0) {
+            return heightIter->toLong();
+        }
+        return 0;
     }
 
     void RafImage::setExifData(const ExifData& /*exifData*/)
@@ -103,12 +109,20 @@ namespace Exiv2 {
         }
         byte const* pData = io_->mmap();
         long size = io_->size();
-        if (size < 84 + 4) throw Error(14); // includes the test for -1
+        if (size < 88 + 4) throw Error(14); // includes the test for -1
         uint32_t const start = getULong(pData + 84, bigEndian) + 12;
         if (static_cast<uint32_t>(size) < start) throw Error(14);
         clearMetadata();
-        TiffParser::decode(this, pData + start, size - start,
-                           TiffCreator::create, TiffDecoder::findDecoder);
+        ByteOrder bo = TiffParser::decode(exifData_,
+                                          iptcData_,
+                                          xmpData_,
+                                          pData + start,
+                                          size - start);
+
+        exifData_["Exif.Image2.JPEGInterchangeFormat"] = getULong(pData + 84, bigEndian);
+        exifData_["Exif.Image2.JPEGInterchangeFormatLength"] = getULong(pData + 88, bigEndian);
+
+        setByteOrder(bo);
     } // RafImage::readMetadata
 
     void RafImage::writeMetadata()
