@@ -20,7 +20,7 @@
  */
 /*
   File:      sigmamn.cpp
-  Version:   $Rev: 1473 $
+  Version:   $Rev: 1512 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   02-Apr-04, ahu: created
   Credits:   Sigma and Foveon MakerNote implemented according to the specification
@@ -29,13 +29,12 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: sigmamn.cpp 1473 2008-06-03 02:15:04Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: sigmamn.cpp 1512 2008-06-21 10:28:17Z ahuggel $")
 
 // *****************************************************************************
 // included header files
 #include "types.hpp"
 #include "sigmamn.hpp"
-#include "makernote.hpp"
 #include "value.hpp"
 #include "i18n.h"                // NLS support.
 
@@ -50,16 +49,6 @@ EXIV2_RCSID("@(#) $Id: sigmamn.cpp 1473 2008-06-03 02:15:04Z ahuggel $")
 // class member definitions
 namespace Exiv2 {
 
-    //! @cond IGNORE
-    SigmaMakerNote::RegisterMn::RegisterMn()
-    {
-        MakerNoteFactory::registerMakerNote("SIGMA", "*", createSigmaMakerNote);
-        MakerNoteFactory::registerMakerNote("FOVEON", "*", createSigmaMakerNote);
-        MakerNoteFactory::registerMakerNote(
-            sigmaIfdId, MakerNote::AutoPtr(new SigmaMakerNote));
-    }
-    //! @endcond
-
     // Sigma (Foveon) MakerNote Tag Info
     const TagInfo SigmaMakerNote::tagInfo_[] = {
         TagInfo(0x0002, "SerialNumber", N_("Serial Number"),
@@ -141,75 +130,9 @@ namespace Exiv2 {
         return tagInfo_;
     }
 
-    SigmaMakerNote::SigmaMakerNote(bool alloc)
-        : IfdMakerNote(sigmaIfdId, alloc)
-    {
-        byte buf[] = {
-            'S', 'I', 'G', 'M', 'A', '\0', '\0', '\0', 0x01, 0x00
-        };
-        readHeader(buf, 10, byteOrder_);
-    }
-
-    SigmaMakerNote::SigmaMakerNote(const SigmaMakerNote& rhs)
-        : IfdMakerNote(rhs)
-    {
-    }
-
-    int SigmaMakerNote::readHeader(const byte* buf,
-                                   long        len,
-                                   ByteOrder   /*byteOrder*/)
-    {
-        if (len < 10) return 1;
-
-        // Copy the header. My one and only Sigma sample has two undocumented
-        // extra bytes (0x01, 0x00) between the ID string and the start of the
-        // Makernote IFD. So we copy 10 bytes into the header.
-        header_.alloc(10);
-        std::memcpy(header_.pData_, buf, header_.size_);
-        // Adjust the offset of the IFD for the prefix
-        start_ = 10;
-        return 0;
-    }
-
-    int SigmaMakerNote::checkHeader() const
-    {
-        int rc = 0;
-        // Check the SIGMA or FOVEON prefix
-        if (   header_.size_ < 10
-            || (   std::string(reinterpret_cast<char*>(header_.pData_), 8)
-                        != std::string("SIGMA\0\0\0", 8)
-                && std::string(reinterpret_cast<char*>(header_.pData_), 8)
-                        != std::string("FOVEON\0\0", 8))) {
-            rc = 2;
-        }
-        return rc;
-    }
-
-    SigmaMakerNote::AutoPtr SigmaMakerNote::create(bool alloc) const
-    {
-        return AutoPtr(create_(alloc));
-    }
-
-    SigmaMakerNote* SigmaMakerNote::create_(bool alloc) const
-    {
-        AutoPtr makerNote(new SigmaMakerNote(alloc));
-        assert(makerNote.get() != 0);
-        makerNote->readHeader(header_.pData_, header_.size_, byteOrder_);
-        return makerNote.release();
-    }
-
-    SigmaMakerNote::AutoPtr SigmaMakerNote::clone() const
-    {
-        return AutoPtr(clone_());
-    }
-
-    SigmaMakerNote* SigmaMakerNote::clone_() const
-    {
-        return new SigmaMakerNote(*this);
-    }
-
     std::ostream& SigmaMakerNote::printStripLabel(std::ostream& os,
-                                                  const Value& value)
+                                                  const Value& value,
+                                                  const ExifData*)
     {
         std::string v = value.toString();
         std::string::size_type pos = v.find(':');
@@ -221,7 +144,8 @@ namespace Exiv2 {
     }
 
     std::ostream& SigmaMakerNote::print0x0008(std::ostream& os,
-                                              const Value& value)
+                                              const Value& value,
+                                              const ExifData*)
     {
         switch (value.toString()[0]) {
         case 'P': os << _("Program"); break;
@@ -234,7 +158,8 @@ namespace Exiv2 {
     }
 
     std::ostream& SigmaMakerNote::print0x0009(std::ostream& os,
-                                              const Value& value)
+                                              const Value& value,
+                                              const ExifData*)
     {
         switch (value.toString()[0]) {
         case 'A': os << _("Average"); break;
@@ -245,16 +170,4 @@ namespace Exiv2 {
         return os;
     }
 
-// *****************************************************************************
-// free functions
-
-    MakerNote::AutoPtr createSigmaMakerNote(bool        alloc,
-                                            const byte* /*buf*/,
-                                            long        /*len*/,
-                                            ByteOrder   /*byteOrder*/,
-                                            long        /*offset*/)
-    {
-        return MakerNote::AutoPtr(new SigmaMakerNote(alloc));
-    }
-
 }                                       // namespace Exiv2
