@@ -20,7 +20,7 @@
  */
 /*
   File:      value.cpp
-  Version:   $Rev: 1421 $
+  Version:   $Rev: 1673 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: value.cpp 1421 2008-05-01 08:22:26Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: value.cpp 1673 2008-11-13 12:28:20Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -43,6 +43,7 @@ EXIV2_RCSID("@(#) $Id: value.cpp 1421 2008-05-01 08:22:26Z ahuggel $")
 #include <cassert>
 #include <cstring>
 #include <ctime>
+#include <cstdio>
 #include <cstdlib>
 #include <ctype.h>
 
@@ -55,6 +56,10 @@ namespace Exiv2 {
     {
     }
 
+    Value::~Value()
+    {
+    }
+
     Value& Value::operator=(const Value& rhs)
     {
         if (this == &rhs) return *this;
@@ -68,10 +73,11 @@ namespace Exiv2 {
         AutoPtr value;
         switch (typeId) {
         case invalidTypeId:
-            value = AutoPtr(new DataValue(invalidTypeId));
-            break;
+        case signedByte:
         case unsignedByte:
-            value = AutoPtr(new DataValue(unsignedByte));
+        case tiffFloat:
+        case tiffDouble:
+            value = AutoPtr(new DataValue(typeId));
             break;
         case asciiString:
             value = AutoPtr(new AsciiValue);
@@ -80,14 +86,12 @@ namespace Exiv2 {
             value = AutoPtr(new ValueType<uint16_t>);
             break;
         case unsignedLong:
+        case tiffIfd:
             value = AutoPtr(new ValueType<uint32_t>);
             break;
         case unsignedRational:
             value = AutoPtr(new ValueType<URational>);
             break;
-        case signedByte:
-            value = AutoPtr(new DataValue(signedByte));
-            break;
         case undefined:
             value = AutoPtr(new DataValue);
             break;
@@ -148,6 +152,21 @@ namespace Exiv2 {
         return toString();
     }
 
+    DataValue::DataValue(TypeId typeId) : Value(typeId)
+    {
+    }
+
+    DataValue::DataValue(const byte* buf,
+              long len, ByteOrder byteOrder,TypeId typeId)
+        : Value(typeId)
+    {
+        read(buf, len, byteOrder);
+    }
+
+    DataValue::~DataValue()
+    {
+    }
+
     int DataValue::read(const byte* buf, long len, ByteOrder /*byteOrder*/)
     {
         // byteOrder not needed
@@ -244,6 +263,7 @@ namespace Exiv2 {
 
     long StringValueBase::copy(byte* buf, ByteOrder /*byteOrder*/) const
     {
+        if (value_.size() == 0) return 0;
         // byteOrder not needed
         assert(buf != 0);
         return static_cast<long>(
@@ -463,7 +483,7 @@ namespace Exiv2 {
         std::ostringstream os;
         write(os);
         std::string s = os.str();
-		std::memcpy(buf, &s[0], s.size());
+        std::memcpy(buf, &s[0], s.size());
         return static_cast<long>(s.size());
     }
 
@@ -618,10 +638,10 @@ namespace Exiv2 {
         return os;
     }
 
-    std::string XmpArrayValue::toString(long n) const 
+    std::string XmpArrayValue::toString(long n) const
     {
         ok_ = true;
-        return value_[n]; 
+        return value_[n];
     }
 
     long XmpArrayValue::toLong(long n) const
@@ -700,12 +720,12 @@ namespace Exiv2 {
         return os;
     }
 
-    std::string LangAltValue::toString(long /*n*/) const 
+    std::string LangAltValue::toString(long /*n*/) const
     {
         return toString("x-default");
     }
 
-    std::string LangAltValue::toString(const std::string& qualifier) const 
+    std::string LangAltValue::toString(const std::string& qualifier) const
     {
         ValueType::const_iterator i = value_.find(qualifier);
         if (i != value_.end()) {
@@ -725,7 +745,7 @@ namespace Exiv2 {
     float LangAltValue::toFloat(long /*n*/) const
     {
         ok_ = false;
-        return 0.0;
+        return 0.0f;
     }
 
     Rational LangAltValue::toRational(long /*n*/) const
@@ -739,11 +759,11 @@ namespace Exiv2 {
         return new LangAltValue(*this);
     }
 
-    DateValue::DateValue() 
+    DateValue::DateValue()
         : Value(date)
     {
     }
-    
+
     DateValue::DateValue(int year, int month, int day)
         : Value(date)
     {
@@ -845,7 +865,7 @@ namespace Exiv2 {
         return l;
     }
 
-    TimeValue::TimeValue() 
+    TimeValue::TimeValue()
         : Value(time)
     {
     }
