@@ -21,7 +21,7 @@
 /*!
   @file    value.hpp
   @brief   Value interface and concrete subclasses
-  @version $Rev: 1421 $
+  @version $Rev: 1698 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    09-Jan-04, ahu: created
@@ -59,7 +59,7 @@ namespace Exiv2 {
       string or data buffer.  For other tasks, like modifying values you may
       need to downcast it to a specific subclass to access its interface.
      */
-    class Value {
+    class EXIV2API Value {
     public:
         //! Shortcut for a %Value auto pointer.
         typedef std::auto_ptr<Value> AutoPtr;
@@ -69,7 +69,7 @@ namespace Exiv2 {
         //! Constructor, taking a type id to initialize the base class with
         explicit Value(TypeId typeId);
         //! Virtual destructor.
-        virtual ~Value() {}
+        virtual ~Value();
         //@}
         //! @name Manipulators
         //@{
@@ -217,6 +217,9 @@ namespace Exiv2 {
           <TR><TD class="indexkey">signedShort</TD><TD class="indexvalue">%ValueType &lt; int16_t &gt;</TD></TR>
           <TR><TD class="indexkey">signedLong</TD><TD class="indexvalue">%ValueType &lt; int32_t &gt;</TD></TR>
           <TR><TD class="indexkey">signedRational</TD><TD class="indexvalue">%ValueType &lt; Rational &gt;</TD></TR>
+          <TR><TD class="indexkey">tiffFloat</TD><TD class="indexvalue">%DataValue(tiffFloat)</TD></TR>
+          <TR><TD class="indexkey">tiffDouble</TD><TD class="indexvalue">%DataValue(tiffDouble)</TD></TR>
+          <TR><TD class="indexkey">tiffIfd</TD><TD class="indexvalue">%ValueType &lt; uint32_t &gt;</TD></TR>
           <TR><TD class="indexkey">date</TD><TD class="indexvalue">%DateValue</TD></TR>
           <TR><TD class="indexkey">time</TD><TD class="indexvalue">%TimeValue</TD></TR>
           <TR><TD class="indexkey">comment</TD><TD class="indexvalue">%CommentValue</TD></TR>
@@ -258,7 +261,7 @@ namespace Exiv2 {
     }
 
     //! %Value for an undefined data type.
-    class DataValue : public Value {
+    class EXIV2API DataValue : public Value {
     public:
         //! Shortcut for a %DataValue auto pointer.
         typedef std::auto_ptr<DataValue> AutoPtr;
@@ -266,14 +269,13 @@ namespace Exiv2 {
         //! @name Creators
         //@{
         //! Default constructor.
-        explicit DataValue(TypeId typeId =undefined) : Value(typeId) {}
+        explicit DataValue(TypeId typeId =undefined);
         //! Constructor
         DataValue(const byte* buf,
                   long len, ByteOrder byteOrder =invalidByteOrder,
-                  TypeId typeId =undefined)
-            : Value(typeId) { read(buf, len, byteOrder); }
+                  TypeId typeId =undefined);
         //! Virtual destructor.
-        virtual ~DataValue() {}
+        virtual ~DataValue();
         //@}
 
         //! @name Manipulators
@@ -330,13 +332,13 @@ namespace Exiv2 {
 
     private:
         //! Internal virtual copy constructor.
-        virtual DataValue* clone_() const;
+        EXV_DLLLOCAL virtual DataValue* clone_() const;
 
     public:
         //! Type used to store the data.
         typedef std::vector<byte> ValueType;
         // DATA
-        ValueType value_;                       //!< Stores the data value 
+        ValueType value_;                       //!< Stores the data value
 
     }; // class DataValue
 
@@ -346,7 +348,7 @@ namespace Exiv2 {
       Uses a std::string to store the value and implements defaults for
       most operations.
      */
-    class StringValueBase : public Value {
+    class EXIV2API StringValueBase : public Value {
     public:
         //! Shortcut for a %StringValueBase auto pointer.
         typedef std::auto_ptr<StringValueBase> AutoPtr;
@@ -432,7 +434,7 @@ namespace Exiv2 {
       left to caller to decode and encode the string to and from readable
       text if that is required.
     */
-    class StringValue : public StringValueBase {
+    class EXIV2API StringValue : public StringValueBase {
     public:
         //! Shortcut for a %StringValue auto pointer.
         typedef std::auto_ptr<StringValue> AutoPtr;
@@ -456,7 +458,7 @@ namespace Exiv2 {
 
     private:
         //! Internal virtual copy constructor.
-        virtual StringValue* clone_() const;
+        EXV_DLLLOCAL virtual StringValue* clone_() const;
 
     }; // class StringValue
 
@@ -466,7 +468,7 @@ namespace Exiv2 {
       This class is for null terminated single byte Ascii strings.
       This class also ensures that the string is null terminated.
      */
-    class AsciiValue : public StringValueBase {
+    class EXIV2API AsciiValue : public StringValueBase {
     public:
         //! Shortcut for a %AsciiValue auto pointer.
         typedef std::auto_ptr<AsciiValue> AutoPtr;
@@ -507,7 +509,7 @@ namespace Exiv2 {
 
     private:
         //! Internal virtual copy constructor.
-        virtual AsciiValue* clone_() const;
+        EXV_DLLLOCAL virtual AsciiValue* clone_() const;
 
     }; // class AsciiValue
 
@@ -519,7 +521,7 @@ namespace Exiv2 {
       undefined), but this is not checked. It is left to caller to decode and
       encode the string to and from readable text if that is required.
     */
-    class CommentValue : public StringValueBase {
+    class EXIV2API CommentValue : public StringValueBase {
     public:
         //! Character set identifiers for the character sets defined by %Exif
         enum CharsetId { ascii, jis, unicode, undefined,
@@ -605,14 +607,14 @@ namespace Exiv2 {
 
     private:
         //! Internal virtual copy constructor.
-        virtual CommentValue* clone_() const;
+        EXV_DLLLOCAL virtual CommentValue* clone_() const;
 
     }; // class CommentValue
 
     /*!
       @brief Base class for all Exiv2 values used to store XMP property values.
      */
-    class XmpValue : public Value {
+    class EXIV2API XmpValue : public Value {
     public:
         //! Shortcut for a %XmpValue auto pointer.
         typedef std::auto_ptr<XmpValue> AutoPtr;
@@ -677,7 +679,7 @@ namespace Exiv2 {
         //@}
 
         /*!
-          @brief Return XMP array type for an array Value TypeId, xaNone if 
+          @brief Return XMP array type for an array Value TypeId, xaNone if
                  \em typeId is not an XMP array value type.
          */
         static XmpArrayType xmpArrayType(TypeId typeId);
@@ -697,13 +699,13 @@ namespace Exiv2 {
     }; // class XmpValue
 
     /*!
-      @brief %Value type suitable for simple XMP properties and 
+      @brief %Value type suitable for simple XMP properties and
              XMP nodes of complex types which are not parsed into
              specific values.
 
       Uses a std::string to store the value.
      */
-    class XmpTextValue : public XmpValue {
+    class EXIV2API XmpTextValue : public XmpValue {
     public:
         //! Shortcut for a %XmpTextValue auto pointer.
         typedef std::auto_ptr<XmpTextValue> AutoPtr;
@@ -722,7 +724,7 @@ namespace Exiv2 {
         /*!
           @brief Read a simple property value from \em buf to set the value.
 
-          Sets the value to the contents of \em buf. A optional keyword, 
+          Sets the value to the contents of \em buf. A optional keyword,
           \em type is supported to set the XMP value type. This is useful for
           complex value types for which Exiv2 does not have direct support.
 
@@ -768,7 +770,7 @@ namespace Exiv2 {
 
     private:
         //! Internal virtual copy constructor.
-        virtual XmpTextValue* clone_() const;
+        EXV_DLLLOCAL virtual XmpTextValue* clone_() const;
 
     public:
         // DATA
@@ -778,14 +780,14 @@ namespace Exiv2 {
 
     /*!
       @brief %Value type for simple arrays. Each item in the array is a simple
-             value, without qualifiers. The array may be an ordered (\em seq), 
-             unordered (\em bag) or alternative array (\em alt). The array 
+             value, without qualifiers. The array may be an ordered (\em seq),
+             unordered (\em bag) or alternative array (\em alt). The array
              items must not contain qualifiers. For language alternatives use
              LangAltValue.
 
       Uses a vector of std::string to store the value(s).
      */
-    class XmpArrayValue : public XmpValue {
+    class EXIV2API XmpArrayValue : public XmpValue {
     public:
         //! Shortcut for a %XmpArrayValue auto pointer.
         typedef std::auto_ptr<XmpArrayValue> AutoPtr;
@@ -800,11 +802,11 @@ namespace Exiv2 {
         //@{
         using XmpValue::read;
         /*!
-          @brief Read a simple property value from \em buf and append it 
+          @brief Read a simple property value from \em buf and append it
                  to the value.
 
           Appends \em buf to the value after the last existing array element.
-          Subsequent calls will therefore populate multiple array elements in 
+          Subsequent calls will therefore populate multiple array elements in
           the order they are read.
 
           @return 0 if successful.
@@ -836,7 +838,7 @@ namespace Exiv2 {
 
     private:
         //! Internal virtual copy constructor.
-        virtual XmpArrayValue* clone_() const;
+        EXV_DLLLOCAL virtual XmpArrayValue* clone_() const;
 
     public:
         //! Type used to store XMP array elements.
@@ -847,12 +849,12 @@ namespace Exiv2 {
     }; // class XmpArrayValue
 
     /*!
-      @brief %Value type for XMP language alternative properties. 
+      @brief %Value type for XMP language alternative properties.
 
       A language alternative is an array consisting of simple text values,
       each of which has a language qualifier.
      */
-    class LangAltValue : public XmpValue {
+    class EXIV2API LangAltValue : public XmpValue {
     public:
         //! Shortcut for a %LangAltValue auto pointer.
         typedef std::auto_ptr<LangAltValue> AutoPtr;
@@ -869,18 +871,18 @@ namespace Exiv2 {
         //@{
         using XmpValue::read;
         /*!
-          @brief Read a simple property value from \em buf and append it 
+          @brief Read a simple property value from \em buf and append it
                  to the value.
 
           Appends \em buf to the value after the last existing array element.
-          Subsequent calls will therefore populate multiple array elements in 
+          Subsequent calls will therefore populate multiple array elements in
           the order they are read.
 
           The format of \em buf is:
           <BR>
           <CODE>[lang=["]language code["] ]text</CODE>
           <BR>
-          The XMP default language code <CODE>x-default</CODE> is used if 
+          The XMP default language code <CODE>x-default</CODE> is used if
           \em buf doesn't start with the keyword <CODE>lang</CODE>.
 
           @return 0 if successful.
@@ -920,7 +922,7 @@ namespace Exiv2 {
 
     private:
         //! Internal virtual copy constructor.
-        virtual LangAltValue* clone_() const;
+        EXV_DLLLOCAL virtual LangAltValue* clone_() const;
 
     public:
         //! Type used to store language alternative arrays.
@@ -930,7 +932,7 @@ namespace Exiv2 {
           @brief Map to store the language alternative values. The language
                  qualifier is used as the key for the map entries.
          */
-        ValueType value_; 
+        ValueType value_;
 
     }; // class LangAltValue
 
@@ -940,7 +942,7 @@ namespace Exiv2 {
       This class is limited to parsing simple date strings in the ISO 8601
       format CCYYMMDD (century, year, month, day).
      */
-    class DateValue : public Value {
+    class EXIV2API DateValue : public Value {
     public:
         //! Shortcut for a %DateValue auto pointer.
         typedef std::auto_ptr<DateValue> AutoPtr;
@@ -956,7 +958,7 @@ namespace Exiv2 {
         //@}
 
         //! Simple Date helper structure
-        struct Date
+        EXIV2API struct Date
         {
             Date() : year(0), month(0), day(0) {}
             int year;                           //!< Year
@@ -1029,7 +1031,7 @@ namespace Exiv2 {
 
     private:
         //! Internal virtual copy constructor.
-        virtual DateValue* clone_() const;
+        EXV_DLLLOCAL virtual DateValue* clone_() const;
 
         // DATA
         Date date_;
@@ -1044,7 +1046,7 @@ namespace Exiv2 {
      seconds and HHMM refers to hours and minutes ahead or behind
      Universal Coordinated Time.
      */
-    class TimeValue : public Value {
+    class EXIV2API TimeValue : public Value {
     public:
         //! Shortcut for a %TimeValue auto pointer.
         typedef std::auto_ptr<TimeValue> AutoPtr;
@@ -1149,7 +1151,7 @@ namespace Exiv2 {
           @param format Format string for sscanf().
           @return 0 if successful, else 1.
          */
-        int scanTime3(const char* buf, const char* format);
+        EXV_DLLLOCAL int scanTime3(const char* buf, const char* format);
         /*!
           @brief Set time from \em buf if it conforms to \em format
                  (6 input items).
@@ -1160,13 +1162,13 @@ namespace Exiv2 {
           @param format Format string for sscanf().
           @return 0 if successful, else 1.
          */
-        int scanTime6(const char* buf, const char* format);
+        EXV_DLLLOCAL int scanTime6(const char* buf, const char* format);
         //@}
 
         //! @name Accessors
         //@{
         //! Internal virtual copy constructor.
-        virtual TimeValue* clone_() const;
+        EXV_DLLLOCAL virtual TimeValue* clone_() const;
         //@}
 
         // DATA
@@ -1209,7 +1211,7 @@ namespace Exiv2 {
         ValueType() : Value(getType<T>()), pDataArea_(0), sizeDataArea_(0) {}
         //! Constructor
         ValueType(const byte* buf, long len, ByteOrder byteOrder);
-        //! Constructor
+        //! Constructor (Note: Argument \em byteOrder is ignored and deprecated)
         explicit ValueType(const T& val, ByteOrder byteOrder =littleEndian);
         //! Copy constructor
         ValueType(const ValueType<T>& rhs);
@@ -1430,12 +1432,10 @@ namespace Exiv2 {
     }
 
     template<typename T>
-    ValueType<T>::ValueType(const T& val, ByteOrder byteOrder)
+    ValueType<T>::ValueType(const T& val, ByteOrder /*byteOrder*/)
         : Value(getType<T>()), pDataArea_(0), sizeDataArea_(0)
     {
-        read(reinterpret_cast<const byte*>(&val),
-             TypeInfo::typeSize(typeId()),
-             byteOrder);
+        value_.push_back(val);
     }
 
     template<typename T>
@@ -1552,6 +1552,7 @@ namespace Exiv2 {
     inline long ValueType<Rational>::toLong(long n) const
     {
         ok_ = (value_[n].second != 0);
+        if (!ok_) return 0;
         return value_[n].first / value_[n].second;
     }
     // Specialization for unsigned rational
@@ -1559,6 +1560,7 @@ namespace Exiv2 {
     inline long ValueType<URational>::toLong(long n) const
     {
         ok_ = (value_[n].second != 0);
+        if (!ok_) return 0;
         return value_[n].first / value_[n].second;
     }
     // Default implementation
@@ -1573,6 +1575,7 @@ namespace Exiv2 {
     inline float ValueType<Rational>::toFloat(long n) const
     {
         ok_ = (value_[n].second != 0);
+        if (!ok_) return 0.0f;
         return static_cast<float>(value_[n].first) / value_[n].second;
     }
     // Specialization for unsigned rational
@@ -1580,6 +1583,7 @@ namespace Exiv2 {
     inline float ValueType<URational>::toFloat(long n) const
     {
         ok_ = (value_[n].second != 0);
+        if (!ok_) return 0.0f;
         return static_cast<float>(value_[n].first) / value_[n].second;
     }
     // Default implementation
