@@ -20,13 +20,13 @@
  */
 /*
   File:      xmp.cpp
-  Version:   $Rev: 1508 $
+  Version:   $Rev: 1703 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   13-July-07, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: xmp.cpp 1508 2008-06-18 15:37:56Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: xmp.cpp 1703 2008-12-15 16:24:37Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -165,7 +165,12 @@ namespace Exiv2 {
 
     std::string Xmpdatum::key() const
     {
-        return p_->key_.get() == 0 ? "" : p_->key_->key(); 
+        return p_->key_.get() == 0 ? "" : p_->key_->key();
+    }
+
+    const char* Xmpdatum::familyName() const
+    {
+        return p_->key_.get() == 0 ? "" : p_->key_->familyName();
     }
 
     std::string Xmpdatum::groupName() const
@@ -225,7 +230,7 @@ namespace Exiv2 {
 
     float Xmpdatum::toFloat(long n) const
     {
-        return p_->value_.get() == 0 ? -1 : p_->value_->toFloat(n); 
+        return p_->value_.get() == 0 ? -1 : p_->value_->toFloat(n);
     }
 
     Rational Xmpdatum::toRational(long n) const
@@ -235,7 +240,7 @@ namespace Exiv2 {
 
     Value::AutoPtr Xmpdatum::getValue() const
     {
-        return p_->value_.get() == 0 ? Value::AutoPtr(0) : p_->value_->clone(); 
+        return p_->value_.get() == 0 ? Value::AutoPtr(0) : p_->value_->clone();
     }
 
     const Value& Xmpdatum::value() const
@@ -250,7 +255,7 @@ namespace Exiv2 {
         return 0;
     }
 
-    std::ostream& Xmpdatum::write(std::ostream& os) const
+    std::ostream& Xmpdatum::write(std::ostream& os, const ExifData*) const
     {
         return XmpProperties::printProperty(os, key(), value());
     }
@@ -273,7 +278,7 @@ namespace Exiv2 {
         if (pValue) p_->value_ = pValue->clone();
     }
 
-    void Xmpdatum::setValue(const std::string& value)
+    int Xmpdatum::setValue(const std::string& value)
     {
         if (p_->value_.get() == 0) {
             TypeId type = xmpText;
@@ -282,7 +287,7 @@ namespace Exiv2 {
             }
             p_->value_ = Value::create(type);
         }
-        p_->value_->read(value);
+        return p_->value_->read(value);
     }
 
     Xmpdatum& XmpData::operator[](const std::string& key)
@@ -329,17 +334,17 @@ namespace Exiv2 {
         std::sort(xmpMetadata_.begin(), xmpMetadata_.end(), cmpMetadataByKey);
     }
 
-    XmpData::const_iterator XmpData::begin() const 
+    XmpData::const_iterator XmpData::begin() const
     {
-        return xmpMetadata_.begin(); 
+        return xmpMetadata_.begin();
     }
 
     XmpData::const_iterator XmpData::end() const
     {
-        return xmpMetadata_.end(); 
+        return xmpMetadata_.end();
     }
 
-    bool XmpData::empty() const 
+    bool XmpData::empty() const
     {
         return count() == 0;
     }
@@ -371,6 +376,8 @@ namespace Exiv2 {
         if (!initialized_) {
 #ifdef EXV_HAVE_XMP_TOOLKIT
             initialized_ = SXMPMeta::Initialize();
+            SXMPMeta::RegisterNamespace("http://www.digikam.org/ns/1.0/", "digiKam", 0);
+            SXMPMeta::RegisterNamespace("http://ns.microsoft.com/Photo/1.0/", "MicrosoftPhoto", 0);
 #else
             initialized_ = true;
 #endif
@@ -390,7 +397,7 @@ namespace Exiv2 {
     }
 
 #ifdef EXV_HAVE_XMP_TOOLKIT
-    bool XmpParser::registerNs(const std::string& ns, 
+    bool XmpParser::registerNs(const std::string& ns,
                                const std::string& prefix)
     {
         try {
@@ -402,7 +409,7 @@ namespace Exiv2 {
         }
     } // XmpParser::registerNs
 #else
-    bool XmpParser::registerNs(const std::string& /*ns*/, 
+    bool XmpParser::registerNs(const std::string& /*ns*/,
                                const std::string& /*prefix*/)
     {
         initialize();
@@ -450,7 +457,7 @@ namespace Exiv2 {
                 continue;
             }
             if (XMP_NodeIsSchema(opt)) {
-                // Register unknown namespaces with Exiv2 
+                // Register unknown namespaces with Exiv2
                 // (Namespaces are automatically registered with the XMP Toolkit)
                 if (XmpProperties::prefix(schemaNs).empty()) {
                     std::string prefix;
