@@ -21,7 +21,7 @@
 /*!
   @file    xmp.hpp
   @brief   Encoding and decoding of XMP data
-  @version $Rev: 1423 $
+  @version $Rev: 1692 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    13-Jul-07, ahu: created
@@ -44,6 +44,10 @@
 // namespace extensions
 namespace Exiv2 {
 
+// *****************************************************************************
+// class declarations
+    class ExifData;
+
 // *****************************************************************************
 // class definitions
 
@@ -51,7 +55,7 @@ namespace Exiv2 {
       @brief Information related to an XMP property. An XMP metadatum consists
              of an XmpKey and a Value and provides methods to manipulate these.
      */
-    class Xmpdatum : public Metadatum {
+    class EXIV2API Xmpdatum : public Metadatum {
     public:
         //! @name Creators
         //@{
@@ -110,17 +114,17 @@ namespace Exiv2 {
           @brief Set the value to the string \em value. Uses Value::read(const
                  std::string&).  If the %Xmpdatum does not have a Value yet,
                  then a %Value of the correct type for this %Xmpdatum is
-                 created. If the key is unknown, a XmpTextValue is used as 
-                 default.
+                 created. If the key is unknown, a XmpTextValue is used as
+                 default. Return 0 if the value was read successfully.
          */
-        void setValue(const std::string& value);
+        int setValue(const std::string& value);
         //@}
 
         //! @name Accessors
         //@{
         //! Not implemented. Calling this method will raise an exception.
         long copy(byte* buf, ByteOrder byteOrder) const;
-        std::ostream& write(std::ostream& os) const;
+        std::ostream& write(std::ostream& os, const ExifData* pMetadata =0) const;
         /*!
           @brief Return the key of the Xmpdatum. The key is of the form
                  '<b>Xmp</b>.prefix.property'. Note however that the
@@ -128,6 +132,7 @@ namespace Exiv2 {
                  contain multiple metadata with the same key.
          */
         std::string key() const;
+        const char* familyName() const;
         //! Return the (preferred) schema namespace prefix.
         std::string groupName() const;
         //! Return the property name.
@@ -171,7 +176,7 @@ namespace Exiv2 {
       - add, modify and delete metadata
       - serialize XMP data to an XML block
     */
-    class XmpData {
+    class EXIV2API XmpData {
     public:
         //! XmpMetadata iterator type
         typedef XmpMetadata::iterator iterator;
@@ -250,7 +255,7 @@ namespace Exiv2 {
              class to parse and serialize XMP packets. The parser uses
              the XMP toolkit to do the job.
      */
-    class XmpParser {
+    class EXIV2API XmpParser {
         friend void XmpProperties::registerNs(const std::string&, const std::string&);
         friend void XmpProperties::unregisterNs(const std::string&);
     public:
@@ -303,7 +308,7 @@ namespace Exiv2 {
         /*!
           @brief Initialize the XMP Toolkit.
 
-          Calling this method is usually not needed, as encode() and 
+          Calling this method is usually not needed, as encode() and
           decode() will initialize the XMP Toolkit if necessary.
 
           @return True if the initialization was successful, else false.
