@@ -20,14 +20,14 @@
  */
 /*
   File:      xmpsidecar.cpp
-  Version:   $Rev: 1458 $
+  Version:   $Rev: 1690 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   07-Mar-08, ahu: created
   Credits:   See header file
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: xmpsidecar.cpp 1458 2008-05-22 08:38:20Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: xmpsidecar.cpp 1690 2008-12-07 10:51:51Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -119,7 +119,7 @@ namespace Exiv2 {
             copyExifToXmp(exifData_, xmpData_);
             copyIptcToXmp(iptcData_, xmpData_);
             if (XmpParser::encode(xmpPacket_, xmpData_,
-                                  XmpParser::omitPacketWrapper|XmpParser::useCompactFormat)) {
+                                  XmpParser::omitPacketWrapper|XmpParser::useCompactFormat) > 1) {
 #ifndef SUPPRESS_WARNINGS
                 std::cerr << "Error: Failed to encode XMP metadata.\n";
 #endif
